#include <math.h>
#include "types.h"

// The algorithm for solving the set of linear equations is taken out of the 
// Numerical Recipies

// Return codes 
// 0  = kSingularMatrix
// 1  = kNoError
// 2  = kMemoryError
#define	kSingularMatrix	0
#define kNoError		1
#define kMemoryError	2
#define TINY 1.0e-20;

int LUDecompose(float **a, int n, int *indx, float *d) ;
int SolveLinearEquations(float **Amatrix,int Ndimension,float *BXsolution) ;
void LUBackSubstitute(float **a, int n, int *indx, float b[]) ;

int SolveLinearEquations(float **Amatrix,int Ndimension,float *BXsolution) {
	int 	*indx ;
	float	d ;
	int	status ;
	
	// Solve the set of linear equations Ax = b
	// Input is the matrix A, the dimension of the set N and the vector b
	// A gets destroyed and the solution is returned in b
	
	indx = new int[Ndimension] ;
	
	if (indx == nil) return kMemoryError ;
	
	// First do the LU Decomposition of the matrix A
	status = LUDecompose(Amatrix,Ndimension,indx,&d) ;
	if (status != kNoError) {
		delete[] indx ;
		return status ;
	}
	
	// Solve the new set of equations
	LUBackSubstitute(Amatrix,Ndimension,indx,BXsolution) ;
	return kNoError ;

}


int LUDecompose(float **a, int n, int *indx, float *d) {
	int i,imax,j,k;
	float big,dum,sum,temp;
	float *vv;

	vv = new float[n] ;
	if (vv == nil) return kMemoryError ;
	
	*d=1.0;
	for (i=0;i<n;i++) {
		big=0.0;
		for (j=0;j<n;j++)
			if ((temp=fabs(a[i][j])) > big) big=temp;
		if (big == 0.0) {
			delete[] vv ;
			return kSingularMatrix ;
		}
		vv[i]=1.0/big;
	}
	for (j=0;j<n;j++) {
		for (i=0;i<j;i++) {
			sum=a[i][j];
			for (k=0;k<i;k++) sum -= a[i][k]*a[k][j];
			a[i][j]=sum;
		}
		big=0.0;
		for (i=j;i<n;i++) {
			sum=a[i][j];
			for (k=0;k<j;k++)
				sum -= a[i][k]*a[k][j];
			a[i][j]=sum;
			if ( (dum=vv[i]*fabs(sum)) >= big) {
				big=dum;
				imax=i;
			}
		}
		if (j != imax) {
			for (k=0;k<n;k++) {
				dum=a[imax][k];
				a[imax][k]=a[j][k];
				a[j][k]=dum;
			}
			*d = -(*d);
			vv[imax]=vv[j];
		}
		indx[j]=imax;
		if (a[j][j] == 0.0) a[j][j]=TINY;
		if (j != n) {
			dum=1.0/(a[j][j]);
			for (i=j+1;i<n;i++) a[i][j] *= dum;
		}
	}
	delete[] vv ;
	return kNoError ;
}

void LUBackSubstitute(float **a, int n, int *indx, float b[]) {
	int i,ii=-1,ip,j;
	float sum;

	for (i=0;i<n;i++) {
		ip=indx[i];
		sum=b[ip];
		b[ip]=b[i];
		if (ii >= 0)
			for (j=ii;j<i;j++) sum -= a[i][j]*b[j];
		else if (sum) ii=i;
		b[i]=sum;
	}
	for (i=n-1;i>=0;i--) {
		sum=b[i];
		for (j=i+1;j<n;j++) sum -= a[i][j]*b[j];
		b[i]=sum/a[i][i];
	}
}

