/*
 * fitting.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "mbconfig.h"
#include "fitting.h"
#include "utils.h"
#include "mb_input.h"
#include "data_out.h"
#include "svd.h"
#include "mb_mpi.h"

extern DEBUG_INFO  dbg;
extern EPS_CTRL  epsilon;

#define BUFLEN  256

/* void read_eps_ctrl_params( FILE *fi ); */


/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
void auto_ed_beams(FIT_EXP_DATASET *ed, CRYSTAL_STRCT_DATA *csd)
{
  /*
    Automatic beam search and allocation of the reflection vector in ed.
   */
  HKL *g_exact, *g_bethe;
  int nr, g_table_size;

  g_table_size = SIZE_GTABLE;
  g_exact = new_vector(HKL, ed->beam_search.nglim);	chk_vector(g_exact);
  g_bethe = new_vector(HKL, ed->beam_search.nglim);	chk_vector(g_bethe);

  if(
    ed->beam_search.flag_bs == 1 ||
    ed->beam_search.flag_bs == 11
    ){/* wg */
    mb_fprintf(stdout, "Beam search (wg) ... ");/*  fflush(stdout); */
    simple_beam_search_wg(
      &(ed->num_exact), &(ed->num_bethe), g_exact, g_bethe,
      ed->beam_search, ed->incidence, ed->zolzbase1, ed->H_tilt, ed->G_tilt,
      ed->accel_kvol, csd
      );
  }
  else if(
    ed->beam_search.flag_bs == 2 ||
    ed->beam_search.flag_bs == 12
    ){	/* sg */
    mb_fprintf(stdout, "Beam search (sg) ... "); /*fflush(stdout);*/
    simple_beam_search_sg(
      &(ed->num_exact), &(ed->num_bethe), g_exact, g_bethe,
      ed->beam_search, ed->incidence, ed->zolzbase1, ed->H_tilt, ed->G_tilt,
      ed->accel_kvol, csd
      );
  }
  else if(
    ed->beam_search.flag_bs == 3 ||
    ed->beam_search.flag_bs == 13
    ){ /* sg - multi for each LZ */
    mb_fprintf(stdout, "Beam search (sg) ... ");/*fflush(stdout);*/
    beam_search_sg_multi(
      &(ed->num_exact), &(ed->num_bethe), g_exact, g_bethe,
      ed->beam_search, ed->incidence, ed->zolzbase1, ed->H_tilt, ed->G_tilt,
      ed->accel_kvol, csd
      );
  }
  else{
    mb_fprintf(
      stdout, "$$ Invalid beam_search.flag_bs. (%d) Abort.\n",
      ed->beam_search.flag_bs
      );
    exit(EXIT_FAILURE);
  }

  mb_fprintf(
    stdout, "$$ Exact = %d, Bethe = %d\n", ed->num_exact, ed->num_bethe
    );
  nr = ed->num_exact + ed->num_bethe;
  ed->reflex = new_vector(HKL, nr);  chk_vector(ed->reflex);
  beam_merge(ed->num_exact, ed->num_bethe, g_exact, g_bethe, ed->reflex);
  free_vector(g_bethe);
  free_vector(g_exact);
}


void calc_K_xy_cbed(
  int *num_k, int *num_k_cal,
  R_HKL k_xy[], HGPIX k_pix[], double H_tilt, double G_tilt,
  UVW incidence, R_HKL zolzbase1, CELL_PAR bv, int calc_type,
  double G_bgn, double G_end, double H_bgn, double H_end, int num_points
  )
{
  /*
   * np: number of points in H (or G) direction --> number of pixels: np * np
   */
  int  i, j, nm, la, ct, ct_cal_points; /* caution: "long" is better? */
  /* int jj; */
  double  bh, bg;
  R_HKL  zolzbase2;

  if (calc_type != 2){
    mb_fprintf(
      stdout, "$$ calc_type must be 2. Abort at calc_k_xy_radial_lnp().\n"
      );
    exit(EXIT_FAILURE);
  }

  zolzbase2 = zolz_ort_base(incidence, zolzbase1, bv);
  nm = num_points - 1;

  ct = 0;  ct_cal_points = 0;
  for(i=0; i<num_points; i++){
    bg = G_tilt + G_bgn + (G_end - G_bgn) * (double) i / (double) nm;
    la =
      2 * (int)(
        sqrt(
          sq((double)(num_points/2) + 0.5) - sq((double)(i-num_points/2))
          )
        );
    /* la = 2 * (int)( sqrt( sq((double)(np/2)) - sq((double)(i-np/2)) ) ); */
    for(j=0; j<num_points; j++){
      bh = H_tilt + H_bgn + (H_end - H_bgn) * (double)j / (double) nm;
      k_xy[ct].h = 0.5 * ( bh * zolzbase1.h + bg * zolzbase2.h );
      k_xy[ct].k = 0.5 * ( bh * zolzbase1.k + bg * zolzbase2.k );
      k_xy[ct].l = 0.5 * ( bh * zolzbase1.l + bg * zolzbase2.l );
      k_pix[ct].ih = j;  k_pix[ct].ig = i;
      k_pix[ct].H = bh;  k_pix[ct].G = bg;
      if ( num_points/2 - la/2 <= j && j <= num_points/2 + la/2 ){
        k_pix[ct].flag = 1; /* intensity calculation */
        ct_cal_points++;
      }
      else{
        k_pix[ct].flag = 0; /* no intensity calculation */
      }
      ct++;
    }
  }
  *num_k = ct; /* ct must be equal to sq(num_points) */
  if ( ct != sq(num_points) ){
    mb_fprintf(stdout, "$$ Wrong ct(=%d) in calc_K_xy_cbed. Abort.\n", ct);
    exit(EXIT_FAILURE);
  }
  *num_k_cal = ct_cal_points;
}


double eval_jc(
  int par_flag, double perturbed_intensity, double intensity, double step
  )
{
  /* Calc. of a jacobian element */
  if( par_flag >= 0 ) {/* fitting parameter */
    return (perturbed_intensity - intensity) / step;
  }
  else{
    return 0.0; /* fix */
  }
}


void eval_jacobian_2D_j(
  int jp, int jthp, double step, FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei,
  int nk, int nph, R_HKL *K_xy, HGPIX *K_pix,
  double ****p_intensity, double ***jacobian
  )
{
  /* Calc. of a jacobian element */
  /* p_intensity[ik][ig][it][ip] */
  int     ig, it, ik, ng, nt, nkobs;
  double  *p, *tmp_k_intensity;

  ng = ed->num_rg; /* number of obs. reflections */
  nt = ed->num_thickness;
  tmp_k_intensity = new_vector(double, nk);  chk_vector(tmp_k_intensity);
  
  for(ig=0; ig<ng; ig++){
    nkobs = ei->num_data_points[ig];
    p = new_vector(double, nkobs);  chk_vector(p);
    for(it=0; it<nt; it++){
      if(jp == jthp){/* search */
        for(ik=0; ik<nk; ik++){
          tmp_k_intensity[ik] = p_intensity[ik][ig][it][jp];
        }
        interpolation_2d(
          p, tmp_k_intensity, K_xy, K_pix,
          ed->H_tilt + ed->H_shift[ig][it],
          ed->G_tilt + ed->G_shift[ig][it],
          ei->obs_H[ig][it], ei->obs_G[ig][it], nkobs, nph, nk,
          ed->num_avrpix[it], ed->avrpix_wt[it]
          );
        for(ik=0; ik<nkobs; ik++){
          jacobian[ig][it][ik]
            = ( p[ik] - ei->cal_intensity[ig][it][ik] ) / step;
        }
      }
      /*
      else {
        for(ik=0; ik<nk; ik++){
          jacobian[par_flag][ig][it][ik] = 0.0;
        }
      }
       */
    }
    free_vector(p);
  }
  free_vector(tmp_k_intensity);
}


void calc_jacobian_pert_j(
  int jthp, CRYSTAL_STRCT_DATA *csd,
  FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei,
  double ****p_intensity, int nksize, int num_points_h,
  R_HKL *K_xy, HGPIX *K_pix,
  int num_fitpar, int num_perturb, double ***jacobian
  )
{
  /*
   * Calc. of jacobian elements for csd parameters
   */
  int i, ia;

  /* p_intensity[ik][ig][it][ip] */

  eval_jacobian_2D_j(
    csd->lp_f.a.ntp, jthp, csd->lp_f.a.dlt,
    ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
    );
  eval_jacobian_2D_j(
    csd->lp_f.b.ntp, jthp, csd->lp_f.b.dlt,
    ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
    );
  eval_jacobian_2D_j(
    csd->lp_f.c.ntp, jthp, csd->lp_f.c.dlt,
    ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
    );
  eval_jacobian_2D_j(
    csd->lp_f.alpha.ntp, jthp, csd->lp_f.alpha.dlt,
    ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
    );
  eval_jacobian_2D_j(
    csd->lp_f.beta.ntp,  jthp, csd->lp_f.beta.dlt,
    ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
    );
  eval_jacobian_2D_j(
    csd->lp_f.gamma.ntp, jthp, csd->lp_f.gamma.dlt,
    ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
    );
  
  for(i=0; i<csd->n_atom; i++){
    eval_jacobian_2D_j(
      csd->atm_f[i].x.ntp, jthp, csd->atm_f[i].x.dlt,
      ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
      );
    eval_jacobian_2D_j(
      csd->atm_f[i].y.ntp, jthp, csd->atm_f[i].y.dlt,
      ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
      );
    eval_jacobian_2D_j(
      csd->atm_f[i].z.ntp, jthp, csd->atm_f[i].z.dlt,
      ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
      );
    if(csd->atm[i].bf == 0){	/* Isotropic */
      eval_jacobian_2D_j(
        csd->atm_f[i].b.ntp, jthp, csd->atm_f[i].b.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
        );
    }
    else if(csd->atm[i].bf == 1){
      eval_jacobian_2D_j(
        csd->atm_f[i].b11.ntp, jthp, csd->atm_f[i].b11.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
        );
      eval_jacobian_2D_j(
        csd->atm_f[i].b22.ntp, jthp, csd->atm_f[i].b22.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
        );
      eval_jacobian_2D_j(
        csd->atm_f[i].b33.ntp, jthp, csd->atm_f[i].b33.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
        );
      eval_jacobian_2D_j(
        csd->atm_f[i].b12.ntp, jthp, csd->atm_f[i].b12.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
        );
      eval_jacobian_2D_j(
        csd->atm_f[i].b23.ntp, jthp, csd->atm_f[i].b23.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
        );
      eval_jacobian_2D_j(
        csd->atm_f[i].b13.ntp, jthp, csd->atm_f[i].b13.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
        );
    }
    eval_jacobian_2D_j(
      csd->atm_f[i].occ.ntp, jthp, csd->atm_f[i].occ.dlt,
      ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
      );
    if(csd->atm[i].af == 0){
      eval_jacobian_2D_j(
        csd->atm_f[i].fabs.ntp, jthp, csd->atm_f[i].fabs.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
        );
    }
    else if(csd->atm[i].af == 1){
      ;
    }
    else if(csd->atm[i].af == 2){
      eval_jacobian_2D_j(
        csd->atm_f[i].db.ntp, jthp, csd->atm_f[i].db.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
        );
    }
    else {
      mb_fprintf(stdout, "$$ Invalid af. Abort.\n");
      exit(EXIT_FAILURE);
    }
  }

  for(i=0; i<csd->num_lofg; i++){
    if( csd->lof_type == 1 ){
      eval_jacobian_2D_j(
        csd->fg_lofg_f[i].re.ntp, jthp, csd->fg_lofg_f[i].re.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian 
        );
      eval_jacobian_2D_j( 
        csd->fg_lofg_f[i].im.ntp, jthp, csd->fg_lofg_f[i].im.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian 
        );
    }
    else if( csd->lof_type == 2 ){
      eval_jacobian_2D_j( 
        csd->fg_lofg_f[i].ela_re.ntp, jthp, csd->fg_lofg_f[i].ela_re.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian 
        );
      eval_jacobian_2D_j( 
        csd->fg_lofg_f[i].ela_im.ntp, jthp, csd->fg_lofg_f[i].ela_im.dlt,
        ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian 
        );
    }
    else if(csd->lof_type == 3 || csd->lof_type == 4){
      for(ia=0; ia<csd->n_atom; ia++){
        eval_jacobian_2D_j(
          csd->fg_lofg_f[i].fe[ia].ntp, jthp, csd->fg_lofg_f[i].fe[ia].dlt,
          ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
          );
        eval_jacobian_2D_j(
          csd->fg_lofg_f[i].fi[ia].ntp, jthp, csd->fg_lofg_f[i].fi[ia].dlt,
          ed, ei, nksize, num_points_h, K_xy, K_pix, p_intensity, jacobian
          );
      }
    }
    else{
      mb_fprintf(stdout, "$$ Invalid lof_type %d. Abort.\n", csd->lof_type);
      exit(EXIT_FAILURE);
    }
  }
  
}


void calc_jacobian_st_j(
  int jthp, FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei,
  double ***intensity, int num_points_h, int nk,
  R_HKL *K_xy, HGPIX *K_pix, double ***jacobian
  )
{
  /* scale & thickness */
  /* intensity[ik][ig][it] */
  int ig, ng, it, nt, ik, nd, jp;
  double *itp_intensity, *tmp_k_intensity;

  ng = ed->num_rg; /* number of obs. reflections */
  nt = ed->num_thickness;
  tmp_k_intensity = new_vector(double, nk);  chk_vector(tmp_k_intensity);

  for(ig=0; ig<ng; ig++){
    nd = ei->num_data_points[ig];
    itp_intensity = new_vector(double, nd);  chk_vector(itp_intensity);
    for(it=0; it<nt; it++){

      /* jacobian for scale factor */
      jp = ed->scale_factor_f[it].ntp;
      if(jp == jthp){
        for(ik=0; ik<nd; ik++){
          jacobian[ig][it][ik]
            = ei->cal_intensity[ig][it][ik] / ed->scale_factor[it];
        }
      }

      /* jacobian for thickness */
      jp = ed->thickness_f[it].ntp;
      if(jp == jthp){
        for(ik=0; ik<nk; ik++){
          tmp_k_intensity[ik] = intensity[ik][ig][it+nt];
        }
        interpolation_2d(
          itp_intensity, tmp_k_intensity, K_xy, K_pix,
          ed->H_tilt + ed->H_shift[ig][it],
          ed->G_tilt + ed->G_shift[ig][it],
          ei->obs_H[ig][it], ei->obs_G[ig][it],
          nd, num_points_h, nk,
          ed->num_avrpix[it], ed->avrpix_wt[it]
          );
        for(ik=0; ik<ei->num_data_points[ig]; ik++){
          jacobian[ig][it][ik]
            = (itp_intensity[ik] - ei->cal_intensity[ig][it][ik])
              / ed->thickness_f[it].dlt;
        }
      }
    }
    free_vector(itp_intensity);
  }
  free_vector(tmp_k_intensity);
}


void calc_jacobian_shift_j(
  int jth_par, FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei, 
  double ***intensity, int nk, int num_points_h,
  R_HKL *K_xy, HGPIX	*K_pix, double ***jacobian
  )
{
  /*
   * Calc. of jacobian elements from num_perturb to (num_fitpar - 1)
   *  for shift
   * Caution: sign of step
   */
  int  ifp, nt, nkobs, ik, it, ig, ng;
  double  step, *shifted_intensity, *tmp_k_intensity;

  ng = ed->num_rg; /* number of obs. reflections */
  nt = ed->num_thickness;
  tmp_k_intensity = new_vector(double, nk);  chk_vector(tmp_k_intensity);

  for(ig=0; ig<ng; ig++){
    nkobs = ei->num_data_points[ig];
    shifted_intensity = new_vector(double, nkobs);
    chk_vector(shifted_intensity);
    for(it=0; it<nt; it++){
      for(ik=0; ik<nk; ik++){
        tmp_k_intensity[ik] = intensity[ik][ig][it];
      }
      /* H shift */
      ifp = ed->H_shift_f[ig][it].ntp;
      if(ifp == jth_par){ /* search */
        step = ed->H_shift_f[ig][it].dlt;
        interpolation_2d(
          shifted_intensity, tmp_k_intensity, K_xy, K_pix,
          ed->H_tilt + ed->H_shift[ig][it] + step,
          ed->G_tilt + ed->G_shift[ig][it],
          ei->obs_H[ig][it], ei->obs_G[ig][it],
          nkobs, num_points_h, nk,
          ed->num_avrpix[it], ed->avrpix_wt[it]
          );
        for(ik=0; ik<nkobs; ik++){
          jacobian[ig][it][ik]
            = ( shifted_intensity[ik] - ei->cal_intensity[ig][it][ik] ) / step;
        }
      }
      /* G shift */
      ifp = ed->G_shift_f[ig][it].ntp;
      if(ifp == jth_par){/* search */
        step = ed->G_shift_f[ig][it].dlt;
        interpolation_2d(
          shifted_intensity, tmp_k_intensity, K_xy, K_pix,
          ed->H_tilt + ed->H_shift[ig][it],
          ed->G_tilt + ed->G_shift[ig][it] + step,
          ei->obs_H[ig][it], ei->obs_G[ig][it],
          nkobs, num_points_h, nk,
          ed->num_avrpix[it], ed->avrpix_wt[it]
          );
        for(ik=0; ik<nkobs; ik++){
          jacobian[ig][it][ik]
            = ( shifted_intensity[ik] - ei->cal_intensity[ig][it][ik] ) / step;
        }
      }

    }
    free_vector(shifted_intensity);
  }
  free_vector(tmp_k_intensity);
}


void calc_jacobian_j(
  int jth_par, CRYSTAL_STRCT_DATA *csd,
  FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei,
  double ***intensity, double ****p_intensity,
  int nk, int num_points_h, int nksize, int num_fitpar, int num_perturb, 
  R_HKL *K_xy, HGPIX	*K_pix, double ***jacobian
  )
{
  /* j-th element of jacobian */
  /* intensity[ik][ig][it], p_intensity[ik][ig][it][ip] */
  calc_jacobian_st_j(/* Scale and thickness */
    jth_par, ed, ei, intensity, num_points_h, nk, K_xy, K_pix, jacobian
    );
  calc_jacobian_pert_j(/* Perturbation: structural parameters */
    jth_par, csd, ed, ei, p_intensity, nksize, num_points_h,
    K_xy, K_pix, num_fitpar, num_perturb, jacobian
    );
  calc_jacobian_shift_j(/* HG shift */
    jth_par, ed, ei, intensity, nksize, num_points_h, K_xy, K_pix, jacobian
    );
}


double cal_weight(
  double obs_sig, double obs_intensity, double cal_intensity,
  double g, double artificial_weight
  )
{
  /*
   * Weight for each intensity data
   *   art_weight: artificial weight for each reflection
   */
  /* 	double weight, p, q; */
  /*	double	sqsig; */
  
  return artificial_weight
    / ( epsilon.w_sq_coef * sq(obs_sig)
        + epsilon.w_exp_coef *  pow(obs_sig, epsilon.w_exp) );
  /*
	sqsig = sq(obs_sig);
	return artificial_weight / ( epsilon.w_sq * sqsig + epsilon.w_4 * sq(sqsig) );
   */
  
#if 0	
  /* (from SHELX-93) */
  if(epsilon.w_c == 0.0){
    q = 1.0;
  }
  else if(epsilon.w_c > 0.0){
    q = exp( epsilon.w_c * sq(g / 2.0) );
  }
  else if(epsilon.w_c < 0.0){
    q = 1.0 - exp( epsilon.w_c * sq(g / 2.0) );
  }
  else{
    mb_fprintf(stdout, "$$ Invalid epsilon.w_c. Abort.\n");
    exit(EXIT_FAILURE);
  }
  
  p = epsilon.w_f * MAX2(0.0, obs_intensity)
    + ( 1.0 - epsilon.w_f ) * cal_intensity;
  weight = art_weight * q / (
    sq(obs_sig) + sq(epsilon.w_a * p) + epsilon.w_b * p
    + epsilon.w_d + epsilon.w_e * g
    );
  return weight;
#endif 
}


double cal_weight_factor(
  double obs_sig, double obs_intensity, double cal_intensity,
  double artificial_weight
  )
{
  /*
   * Weight for each intensity data
   *   art_weight: artificial weight for each reflection
   */
  return artificial_weight / sq(obs_sig);
/*  return artificial_weight /
    (
      epsilon.w_sq_coef * sq(obs_sig) +
      epsilon.w_exp_coef * pow(obs_sig, epsilon.w_exp)
      ); */
}


void calc_alpha(
  FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei, 
  int ip1, int ip2, double ***jacobian1, double ***jacobian2,
  int num_fitpar, double **alpha
  )
{
  int it, nt, ig, ng, ik, nk;
  /*	int	l, ll; */
  double dy, obs_sig, obs_int, cal_int, wt, sig2i, g, aw, t;
  
  /* ----- Evaluation of alpha[][] ----- */
  nt = ed->num_thickness;
  ng = ed->num_rg;
  t = 0.0;
  
  for(it=0; it<nt; it++){
    for(ig=0; ig<ng; ig++){
      g = ed->g[ig];
      aw = ed->art_weight[ig];
      nk = ei->num_data_points[ig];
      for(ik=0; ik<nk; ik++){
        obs_sig = ei->obs_sig[ig][it][ik];
        obs_int = ei->obs_intensity[ig][it][ik];
        cal_int = ei->cal_intensity[ig][it][ik];
        sig2i = cal_weight( obs_sig, obs_int, cal_int, g, aw );
        dy = obs_int - cal_int;

        if(dbg.obs_cal_dump_flag == 1){/* debug */
          mb_fprintf(stdout, "$$ obs, cal = %lf, %lf\n", obs_int, cal_int);
        }

        wt = jacobian1[ig][it][ik] * sig2i;
        t += wt * jacobian2[ig][it][ik];
      }
    }
  }
  alpha[ip1][ip2] += t;
}


void calc_beta(
  FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei, int ip1, double ***jacobian1,
  int num_fitpar, double *beta
  )
{
  /*
   * Evaluation of beta[]
   */
  int it, nt, ig, ng, ik, nk;
  /* int  ll; */
  double dy, obs_sig, obs_int, cal_int, wt, sig2i, g, aw, t;

  nt = ed->num_thickness;
  ng = ed->num_rg;
  t = 0.0;
  for(ig=0; ig<ng; ig++){
    for(it=0; it<nt; it++){
      g = ed->g[ig];
      aw = ed->art_weight[ig];
      nk = ei->num_data_points[ig];
      for(ik=0; ik<nk; ik++){
        obs_sig = ei->obs_sig[ig][it][ik];
        obs_int = ei->obs_intensity[ig][it][ik];
        cal_int = ei->cal_intensity[ig][it][ik];
        sig2i = cal_weight( obs_sig, obs_int, cal_int, g, aw );
        dy = obs_int - cal_int;
        wt = jacobian1[ig][it][ik] * sig2i;
        t += dy * wt;  /* Beta */
      }
    }
  }
  beta[ip1] += t;
}


void nksize_evaluation(
  FIT_EXP_DATASET *ed, double *G_bgn, double *G_end,
  double *H_bgn, double *H_end, int *num_points_h
  )
{
  if(ed->calc_range_flag == 0){	/* manual setting */
    *H_bgn = ed->calc_hbgn;
    *H_end = ed->calc_hend;
    *G_bgn = ed->calc_gbgn;
    *G_end = ed->calc_gend;
    *num_points_h = ed->calc_numh;
  }
  /*
   else if(epsilon.cal_range_flag == 1){
    ----- Automatic optimization of calculation points -----
   }
   */
  else{
    mb_fprintf(
      stdout,
      "$$ cal_range_flag=%d is invalid. Abort.", epsilon.cal_range_flag
      );
    exit(EXIT_FAILURE);
  }
}


void perturbed_potential_matrix(
  int num_perturb, complex ***perturb_vgs,
  CRYSTAL_STRCT_DATA *csd, FIT_EXP_DATASET *ed, CELL_PAR bv
  )
{
  CRYSTAL_STRCT_DATA csd_perturb;
  int  ip, nr;
  complex  **vg;

  nr = ed->num_exact + ed->num_bethe;
  vg = new_matrix(complex, nr, nr);
  alloc_csd(
    csd->num_sym_op, csd->n_atom, csd->num_lofg, &csd_perturb
    );
  alloc_csd_sub_cstr(
    csd->n_atom, csd->num_lofg, csd->lof_type, &csd_perturb
    );

  for(ip=0; ip<num_perturb; ip++){
    cpy_csd(csd, &csd_perturb);
    make_csd_perturb( ip, csd, &csd_perturb );
    potential_matrix_csd(
      ed->num_exact, ed->num_bethe, ed->reflex,
      &csd_perturb, ed->accel_kvol, bv, vg
      );
    /*
    fprintf(stdout, "perturbation vg: ip=%d\n", ip);
    fprintf(stdout, "Vg: [Volt]\n");
    fprint_complexmat(stdout, ed->num_exact+ed->num_bethe, vg);
     */
    vg_to_pvg( ip, nr, vg, perturb_vgs );
  }
  free_matrix(vg);
  free_csd(&csd_perturb);
}


void calc_cbed_pattern_perturb(
  CRYSTAL_STRCT_DATA *csd, FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei,
  complex **vg, int num_perturb, complex ***perturbed_vgs,
  int num_fitpar, double **alpha, double *beta, double *chisq,
  FILE *fo_log
  )
{
  int      ng, nt, nr, ig, ik, it, nk, ip, tmp_num_thickness, *tmp_obs_rg;
  int      num_points_h, nksize, oldpix, ip1, ip2, nk_cal;
  int      exec_time_flag;
  double   abs_incidence, abs_surf_norm, cosfai_inc_surf, wave_length;
  double   ev_dash, abs_K;
  double   ***intensity, *itp_intensity, ****p_intensity, tmp;
  double   *tmp_k_intensity;
  double   *t_thickness, *thickness_weight, ***jacobian1, ***jacobian2;
  double   *tmp_thickness, **tmp_intensity, ***tmp_perturb_intensity;
  double   G_bgn, G_end, H_bgn, H_end, g, aw;
  double   cal_int, obs_int, obs_sig, sig2i, dy;
  R_HKL	*K_xy;
  HGPIX	*K_pix;
  CELL_PAR	bv;
  WORK_CAL_INT wci;

#if defined(MPI_MODE)
  MPI_Status  mpistatus;
  int  j, a, b, multp, mulgt, mulgtp, *start, *end;
  double  *tmp1output, *tmp2output;
  double  time_start, time_finish;
#endif

  bv = bravai(csd->lp);
  pre_calc_intensity(
    ed->accel_kvol, ed->incidence, ed->surf_norm, bv,
    &abs_incidence, &abs_surf_norm, &cosfai_inc_surf, &wave_length, &ev_dash
    );

  ng = ed->num_rg;	/* number of obs. reflections */
  nt = ed->num_thickness;
  nr = ed->num_exact + ed->num_bethe;

  tmp_obs_rg = new_vector(int, ng);
  chk_vector(tmp_obs_rg);
  tmp_num_thickness = 2 * nt * (epsilon.num_thickwidth * 2 + 1);
  tmp_thickness = new_vector(double, tmp_num_thickness);
  chk_vector(tmp_thickness);
  t_thickness   = new_vector(double, nt * 2);
  chk_vector(t_thickness);
  thickness_weight
    = new_vector(double, epsilon.num_thickwidth * 2 + 1);
  chk_vector(thickness_weight);
  thickweight_set( epsilon.num_thickwidth, thickness_weight );
  thickness_set( ed, epsilon.num_thickwidth, tmp_thickness ); 

  /* temporaty args. */
  tmp_intensity = new_matrix(double, ng, tmp_num_thickness);
  chk_matrix(tmp_intensity);
  if(num_perturb > 0){
    tmp_perturb_intensity
      = new_tensor(double, num_perturb, ng, tmp_num_thickness);
    chk_tensor(tmp_perturb_intensity);
  }
  alloc_work_cal_int(nr, ng, num_perturb, &wci);

  abs_K = calc_abs_K( csd, ed->accel_kvol, bv );

  nksize_evaluation(ed, &G_bgn, &G_end, &H_bgn, &H_end, &num_points_h);
  nksize = sq(num_points_h);

/*  intensity  = new_tensor(double, ng, tmp_num_thickness, nksize); */
  intensity  = new_tensor(double, nksize, ng, tmp_num_thickness); /* 030728 */
  chk_tensor(intensity);
  if(num_perturb > 0){
    /* 030728 */
    p_intensity
      = new_tensor4(double, nksize, ng, tmp_num_thickness, num_perturb);
    chk_tensor4(p_intensity);
  }
  K_xy = new_vector(R_HKL, nksize);   chk_vector(K_xy);
  K_pix = new_vector(HGPIX, nksize);  chk_vector(K_pix);

  calc_K_xy_cbed(
    &nk, &nk_cal, K_xy, K_pix, ed->H_tilt, ed->G_tilt,
    ed->incidence, ed->zolzbase1, bv, ed->calc_type,
    G_bgn, G_end, H_bgn, H_end, num_points_h
    );
  /* Caution: nk must be equal not to
   ei[ig]->num_data_points but sq(num_points_h) */

  for(ig=0; ig<ng; ig++){
    tmp_obs_rg[ig]
      = find_n_g(
        ed->num_exact + ed->num_bethe, ei->obs_rg[ig], ed->reflex
        );
    tmp = sq_abs(hkl2rhkl(ei->obs_rg[ig]), bv);	ed->g[ig] = sqrt(tmp);
    if(tmp_obs_rg[ig] == -1){
      mb_fprintf(
        stdout,
        "$$ calc_cbed_pattern_perturb: %d %d %d not found in reflex[]. Abort.",
        ei->obs_rg[ig].h, ei->obs_rg[ig].k, ei->obs_rg[ig].l
        );
      exit(EXIT_FAILURE);
    }
  }
  mb_fprintf(stdout, "**CBED intensity calculation\n");
  mb_fprintf(
    stdout, "$$ nk = %d x %d = %ld\n", num_points_h, num_points_h, nk
    );
  exec_time_flag = 0;

#if defined(MPI_MODE)
  /*
     MPI Calc Area
   */
  mb_fprintf(
    stdout, "\n[%d] **** Starting MPI Calculation-Area Setup ****\n", proc_id
    );
  a = nk / num_procs;
  b = nk % num_procs;
  multp = tmp_num_thickness * num_perturb;
  mulgt = ng * tmp_num_thickness;
  mulgtp = ng * tmp_num_thickness * num_perturb;
  start = (int *)malloc( sizeof(int) * num_procs);
  end   = (int *)malloc( sizeof(int) * num_procs);
  tmp1output = (double *)malloc( sizeof(double) * nk * mulgt * 2);
  if(num_perturb > 0){
    tmp2output = (double *)malloc( sizeof(double) * nk * mulgtp);
  }
  start[0] = 0;
  end[0] = a;
  if ( b > 0 ) end[0]++;
  for ( j = 1; j < num_procs; j++){
    start[j] = end[j-1];
    end[j] = start[j] + a;
    if ( b > j ) end[j]++;
  }
  mb_fprintf(
    stdout, "\n[%d] **** Ending MPI Calculation-Area Setup ****\n", proc_id
    );
#endif

#if defined(MPI_MODE)
  MPI_Barrier( MPI_COMM_WORLD );
  time_start = MPI_Wtime();
  mb_fprintf(
    stdout, "\n[%d] **** Starting MPI Diagonalization ****\n", proc_id
    );
  for(ik = start[proc_id]; ik < end[proc_id]; ik++){
#else
  for(ik=0; ik<nk; ik++){
#endif
    if(K_pix[ik].flag == 1){
      if(exec_time_flag==0) {
        exec_time_forecast(-1);
        mb_fprintf(stdout, " [for 1 cycle] ");
      }
      progress_indicator(ik, nk);
      calc_intensity_with_perturb_ug(
        K_xy[ik], abs_K, ed->beam_search, ed->num_exact, ed->num_bethe,
        ed->reflex, vg, num_perturb, perturbed_vgs, ng, tmp_obs_rg,
        tmp_num_thickness, tmp_thickness,
        ed->accel_kvol, ed->incidence, ed->surf_norm, bv,
        abs_incidence, abs_surf_norm, cosfai_inc_surf,
        tmp_intensity, tmp_perturb_intensity, &wci
        );
      nt = ed->num_thickness;
      for(ig=0; ig<ng; ig++){
        for(it=0; it<nt; it++){
          /* Scaling and Averaging */
          scale_and_average_2d(
            epsilon.num_thickwidth, ed->scale_factor[it], ed->thickwidth[it],
            it, nt, ig, ik, num_perturb, thickness_weight, 
            tmp_intensity, tmp_perturb_intensity, intensity, p_intensity
            );
        }
      }
      if(exec_time_flag==0){
        exec_time_forecast(nk_cal);
        exec_time_flag = 1;
      }
    }
    else{
      progress_indicator(ik, nk);
      nt = ed->num_thickness;
      for(ig=0; ig<ng; ig++){
        for(it=0; it<nt; it++){
          intensity[ik][ig][it]    = 0.0;
          intensity[ik][ig][it+nt] = 0.0; /* gradient for thickness */
          for(ip=0; ip<num_perturb; ip++){
            p_intensity[ik][ig][it][ip] = 0.0;
          }
        }
      }
    }
  }
  exec_time_forecast(0);

#if defined(MPI_MODE)
  mb_fprintf(stdout, "\n[%d] **** Ending MPI Diagonalization ****\n", proc_id);
#endif

#if defined(MPI_MODE)
  if(num_procs > 1){
    for( ik = start[proc_id]; ik < end[proc_id]; ik++ ){
      for(ig=0; ig<ng; ig++){
        for(it=0; it<tmp_num_thickness; it++){
          tmp1output[ik * mulgt
                     + ig * tmp_num_thickness + it]   /* Caution 030728 */
            = intensity[ik][ig][it];
        }
      }
    }
    if(num_perturb > 0){
      for( ik = start[proc_id]; ik < end[proc_id]; ik++ ){
        for(ig=0; ig<ng; ig++){
          for(it=0; it<tmp_num_thickness; it++){
            for(ip=0; ip<num_perturb; ip++){
              /*
              tmp2output[ik * mulgtp
                         + ip * mulgt
                         + ig * tmp_num_thickness + it] */  /* 030728 */
              tmp2output[ik * mulgtp + ig * multp + it * num_perturb + ip]
                = p_intensity[ik][ig][it][ip];              /* 031002 */
            }
          }
        }
      }
    }
    MPI_Barrier( MPI_COMM_WORLD );
    mb_fprintf(
      stdout, "\n\t[%d] **** Starting MPI Communication ****\n", proc_id               );
    if( proc_id == 0 ){
      for (j=1; j<num_procs; j++){
        MPI_Recv(
          tmp1output + start[j] * mulgt,
          (end[j] - start[j]) * mulgt,
          MPI_DOUBLE, j, 100, MPI_COMM_WORLD, &mpistatus
          );
      }
    }
    else{
      MPI_Ssend(
        tmp1output + start[proc_id] * mulgt,
        (end[proc_id] - start[proc_id]) * mulgt,
        MPI_DOUBLE, 0, 100, MPI_COMM_WORLD
        );
    }
    if( proc_id == 0 ){
      for (j=1; j<num_procs; j++){
        MPI_Ssend(
          tmp1output, nk * mulgt,
          MPI_DOUBLE, j, 100, MPI_COMM_WORLD
          );
      }
    }
    else{
      MPI_Recv(
        tmp1output, nk * mulgt,
        MPI_DOUBLE, 0, 100, MPI_COMM_WORLD, &mpistatus
        );
    }
    if(num_perturb > 0){
      if( proc_id == 0 ){
        for (j=1; j<num_procs; j++){
          MPI_Recv(
            tmp2output + start[j] * mulgtp,
            (end[j] - start[j]) * mulgtp,
            MPI_DOUBLE, j, 100, MPI_COMM_WORLD, &mpistatus
            );
        }
      }
      else{
        MPI_Ssend(
          tmp2output + start[proc_id] * mulgtp,
          (end[proc_id] - start[proc_id]) * mulgtp,
          MPI_DOUBLE, 0, 100, MPI_COMM_WORLD
          );
      }
      if( proc_id == 0 ){
        for ( j = 1; j < num_procs; j++){
          MPI_Ssend(
            tmp2output, nk * mulgtp,
            MPI_DOUBLE, j, 100, MPI_COMM_WORLD
            );
        }
      }
      else{
        MPI_Recv(
          tmp2output, nk * mulgtp,
          MPI_DOUBLE, 0, 100, MPI_COMM_WORLD, &mpistatus
          );
      }
    }
    for(ik = 0; ik < nk; ik++){
      for(ig=0; ig<ng; ig++){
        for(it=0; it<tmp_num_thickness; it++){
          intensity[ik][ig][it]
            = tmp1output[ik * mulgt
                         + ig * tmp_num_thickness + it];  /* Caution 030728 */
        }
      }
    }
    if(num_perturb > 0){
      for(ik = 0; ik < nk; ik++){
        for(ig=0; ig<ng; ig++){
          for(it=0; it<tmp_num_thickness; it++){
            for(ip=0; ip<num_perturb; ip++){
              p_intensity[ik][ig][it][ip]
                /* = tmp2output[
                  ik * mulgtp + ip * mulgt + ig * tmp_num_thickness + it
                  ];*/  /* 030728 */
                = tmp2output[ik * mulgtp + ig * multp + it * num_perturb + ip];
                /* 031002 */
            }
          }
        }
      }
    }
  }
  free(start);
  free(end);
  free(tmp1output);
  if(num_perturb > 0){ free(tmp2output); }
  MPI_Barrier( MPI_COMM_WORLD );
  time_finish = MPI_Wtime();
  mb_fprintf(
    stdout, "\n[%d] **** Ending MPI Communication ****\n", proc_id
    );
  time_finish -= time_start;
  if( proc_id == 0 ){
    mb_fprintf(
      stdout, "\n\nElapsed time = %e seconds = %lf hours\n\n",
      time_finish, time_finish / 3600.0
      );
    mb_fprintf(
      fo_log, "\n\nElapsed time = %e seconds = %lf hours\n\n",
      time_finish, time_finish / 3600.0
      );
  }
#endif
  
  /* Transform cal into obs by interpolation and calc. chisq */
  tmp_k_intensity = new_vector(double, nk);  chk_vector(tmp_k_intensity);
  for(ig=0; ig<ng; ig++){
    itp_intensity = new_vector(double, ei->num_data_points[ig]);
    chk_vector(itp_intensity);
    g = ed->g[ig];
    aw = ed->art_weight[ig];
    for(it=0; it<nt; it++){
      for(ik=0; ik<nk; ik++){
        tmp_k_intensity[ik] = intensity[ik][ig][it];
      }
      interpolation_2d(
        itp_intensity, tmp_k_intensity, K_xy, K_pix,
        ed->H_tilt + ed->H_shift[ig][it], ed->G_tilt + ed->G_shift[ig][it],
        ei->obs_H[ig][it], ei->obs_G[ig][it],
        ei->num_data_points[ig], num_points_h, nk,
        ed->num_avrpix[it], ed->avrpix_wt[it]
        );          /* <<<<< */
      for(ik=0; ik<ei->num_data_points[ig]; ik++){
        cal_int = ei->cal_intensity[ig][it][ik] = itp_intensity[ik];
        obs_sig = ei->obs_sig[ig][it][ik];
        obs_int = ei->obs_intensity[ig][it][ik];
        sig2i = cal_weight(obs_sig, obs_int, cal_int, g, aw);
        dy = obs_int - cal_int;
        (*chisq) += sq(dy) * sig2i;     /* Chi square */
        if(dbg.obs_cal_dump_flag == 1){ /* debug */
          mb_fprintf( stdout, "$$ obs, cal = %lf, %lf\n", obs_int, cal_int );
        }
      }
    }
    free_vector(itp_intensity);
  }
  free_vector(tmp_k_intensity);
  
  /* Calc. of Jacobian, alpha (upper half), and beta */
  jacobian1 = alloc_jacobian_j(
    ed->num_rg, ed->num_thickness, ei->num_data_points
    );
  jacobian2 = alloc_jacobian_j(
    ed->num_rg, ed->num_thickness, ei->num_data_points
    );

  mb_fprintf(stdout, "**Jacobian calc. 0-%d:\n", num_fitpar-1);
  exec_time_forecast(-1);
  mb_fprintf(stdout, "failed.\n");
  for(ip1=0; ip1<num_fitpar; ip1++){
    progress_indicator(ip1, num_fitpar);
    /* mb_fprintf(stdout, "$$%d",ip1); */
    init_jacobian_j(
      ed->num_rg, ed->num_thickness, ei->num_data_points, jacobian1
      );
    calc_jacobian_j(
      ip1, csd, ed, ei, intensity, p_intensity,
      nk, num_points_h, nksize, num_fitpar, num_perturb, K_xy, K_pix, jacobian1
      );
    calc_beta( ed, ei, ip1, jacobian1, num_fitpar, beta);
    for(ip2=0; ip2<=ip1; ip2++){
      init_jacobian_j(
        ed->num_rg, ed->num_thickness, ei->num_data_points, jacobian2
        );
      calc_jacobian_j(
        ip2, csd, ed, ei, intensity, p_intensity,
        nk, num_points_h, nksize, num_fitpar, num_perturb, K_xy, K_pix,
        jacobian2
        );
      calc_alpha( ed, ei, ip1, ip2, jacobian1, jacobian2, num_fitpar, alpha );
    }
  }
  exec_time_forecast(0);

  free_jacobian_j(jacobian1);
  free_jacobian_j(jacobian2);

  free_vector(K_xy);
  free_vector(K_pix);
  free_tensor(intensity);
  if(num_perturb > 0){
    free_tensor4(p_intensity);
    /* free_vector(tmp_perturb_intensity);*/
    free_tensor(tmp_perturb_intensity);
  }
  free_work_cal_int(&wci);
  free_vector(tmp_obs_rg);
  free_vector(tmp_thickness);
  free_vector(tmp_intensity);
  free_vector(thickness_weight);
  free_vector(t_thickness);
}


void dynamical_calculation(
  CRYSTAL_STRCT_DATA *csd,
  int ith_expdata, FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei,
  int num_fitpar, int num_perturb,
  double **alpha, double *beta, double *chisq, FILE *fo_log
  )
{
  int  nr;
  complex  **vg, ***perturb_vgs;
  CELL_PAR  bv;

  /* ----- Beam search ----- */
  if( ed->beam_search.flag_bs != 0 && ed->beam_search.flag_bs != 10 ){
/*    if(
    ed->beam_search.flag_bs == 1 || ed->beam_search.flag_bs == 11 ||
    ed->beam_search.flag_bs == 2 || ed->beam_search.flag_bs == 12 ||
    ed->beam_search.flag_bs == 3 || ed->beam_search.flag_bs == 13
    ){ */
    auto_ed_beams(ed, csd);
  }

  bv = bravai(csd->lp);
  reflex_dump_csd(
    ith_expdata, ed->num_exact, ed->num_bethe, ed->reflex, csd,
    ed->beam_search, ed->incidence, ed->zolzbase1, ed->accel_kvol, bv
    );

  /* ----- Potential matrix ----- */
  nr = ed->num_exact + ed->num_bethe;
  vg = new_matrix(complex, nr, nr);	chk_matrix(vg);
  mb_fprintf(stdout, "**Potential :\n");
  potential_matrix_csd(
    ed->num_exact, ed->num_bethe, ed->reflex, csd, ed->accel_kvol, bv, vg
    );
  debug_vg_dump( vg, ed->num_exact + ed->num_bethe );
/*
  fprintf(stdout, "Unperturbed vg:\n");
  fprintf(stdout, "Vg: [Volt]\n");
  fprint_complexmat(stdout, ed->num_exact+ed->num_bethe, vg);
*/

  if(num_perturb > 0){
    /* Perturbed potential matrices */
    mb_fprintf(stdout, "**Perturbed potentials:\n");	mb_fflush(stdout);
    perturb_vgs = new_tensor(complex, num_perturb, nr, nr);
    chk_tensor(perturb_vgs);
    perturbed_potential_matrix( num_perturb, perturb_vgs, csd, ed, bv );
/*
    fprintf(stdout, "Perturbed vg:\n");
    fprintf(stdout, "Vg: [Volt]\n");
    fprint_complexmat(stdout, ed->num_exact+ed->num_bethe, perturb_vgs[0]);
*/
  }

  /* ----- Calc. intensity ----- */
  if(
    ed->calc_type == 0 || ed->calc_type == 1 ||
    ed->calc_type == 2 || ed->calc_type == 3
    ){
    calc_cbed_pattern_perturb(
      csd, ed, ei, vg, num_perturb, perturb_vgs, num_fitpar,
      alpha, beta, chisq, fo_log
      );
  }
  else{
    mb_fprintf(stdout, "$$ Invalid calc_type %d\n", ed->calc_type);
    exit(EXIT_FAILURE);
  }

  free_matrix(vg);
  if(num_perturb > 0){ free_tensor(perturb_vgs); }
  if( ed->beam_search.flag_bs != 0 && ed->beam_search.flag_bs != 10 ){
    free_vector(ed->reflex);
  }
}


void gaussj(double **a, int n, double **b, int m)
{
  int  *indxc, *indxr, *ipiv;
  int  i, icol, irow, j, k, l, ll;
  double  big, dum, pivinv, temp;

  indxc=new_vector(int,n); chk_vector(indxc);
  indxr=new_vector(int,n); chk_vector(indxr);
  ipiv =new_vector(int,n); chk_vector(ipiv);

  for (j=0;j<n;j++) {ipiv[j]=0;}
  for (i=0;i<n;i++) {
    big=0.0;
    for (j=0; j<n; j++){
      if (ipiv[j] != 1){
        for (k=0; k<n; k++) {
          if (ipiv[k] == 0) {
            if (fabs(a[j][k])>=big) {
              big=fabs(a[j][k]);
              irow=j;
              icol=k;
            }
          }
          else if (ipiv[k] > 1) {
            mb_fprintf(stdout, "$$ gaussj: Singular Matrix-1\n");
          }
        }
      }
    }
    ++(ipiv[icol]);
    if (irow != icol) {
      for (l=0;l<n;l++){
        /* SWAP(a[irow][l],a[icol][l]) */
        temp = a[irow][l];
        a[irow][l] = a[icol][l];
        a[icol][l] = temp;
        }
      for (l=0;l<m;l++) {
        /* SWAP(b[irow][l],b[icol][l]) */
        temp = b[irow][l];
        b[irow][l] = b[icol][l];
        b[icol][l] = temp;
      }
    }
    indxr[i]=irow;
    indxc[i]=icol;
    if (a[icol][icol] == 0.0){
      mb_fprintf(stdout, "$$ gaussj: Singular Matrix-2\n");
    }
    pivinv=1.0 / a[icol][icol];
    a[icol][icol]=1.0;
    for (l=0;l<n;l++) {a[icol][l] *= pivinv;}
    for (l=0;l<m;l++) {b[icol][l] *= pivinv;}
    for (ll=0;ll<n;ll++){
      if (ll != icol) {
        dum = a[ll][icol];
        a[ll][icol] = 0.0;
        for (l=0;l<n;l++) {a[ll][l] -= a[icol][l]*dum;}
        for (l=0;l<m;l++) {b[ll][l] -= b[icol][l]*dum;}
      }
    }
  }
  for (l=n-1;l>=0;l--) {
    if (indxr[l] != indxc[l])
      for (k=0;k<n;k++){
        /*SWAP(a[k][indxr[l]],a[k][indxc[l]]); */
        temp = a[k][indxr[l]];
        a[k][indxr[l]] = a[k][indxc[l]];
        a[k][indxc[l]] = temp;
    }
  }
  free_vector(ipiv);
  free_vector(indxr);
  free_vector(indxc);
}


void marquardt_coef(
  CRYSTAL_STRCT_DATA *csd,
  int num_exp_dataset, FIT_EXP_DATASET *fed, FIT_EXP_INTSET *fei, 
  int num_fitpar, int num_perturb, double **alpha, double *beta,
  double *chisq, FILE *fo_log
  )
{
  int j, ie, k, l, ll;
  
  for (j=0; j<num_fitpar; j++) {/* Initialize */
    for (k=0; k<=j; k++) {
      alpha[j][k]=0.0;
    }
    beta[j]=0.0;
  }

  /* constraint_csd(csd); */
  constraint_fed(num_exp_dataset, fed);

  *chisq=0.0;
  for(ie=0; ie<num_exp_dataset; ie++){ /* For each set of experimental data */
    dynamical_calculation(
      csd, ie, &(fed[ie]), &(fei[ie]),
      num_fitpar, num_perturb, alpha, beta, chisq, fo_log
      );
  }

  for(l=1; l<num_fitpar; l++){ /* Fill the remaining lower half of alpha */
    for(ll=0; ll<=l-1; ll++){
      alpha[ll][l] = alpha[l][ll];
    }
  }

  dbg_print_dmat(dbg.alpha_dump_flag, "***** alpha:", num_fitpar, alpha);
  dbg_print_dvec(dbg.beta_dump_flag,  "***** beta:", num_fitpar, beta);
}


void marquardt_min(
  CRYSTAL_STRCT_DATA *csd, int num_exp_dataset,
  FIT_EXP_DATASET *fed, FIT_EXP_INTSET *fei,
  char *infile, int maxnum_iter, int num_fitpar, int num_perturb,
  double **covar, double **alpha, double *chisq, double *alamda,
  int *sf_flag, FILE *fo_log
  )
{
  /*
    Levenberg-Marquardt method
     *sf_flag = 0:fail, 1:succeed
   */
  int  i, k, j;
  static double  *da, *atry, **one_da, *beta, old_chisq, *fitpar;
  char  int_outfile[BUFLEN];
  static CRYSTAL_STRCT_DATA  csd_try;
  static FIT_EXP_DATASET  *fed_try;

  if (*alamda < 0.0) {/* Initital cycle */
    one_da = new_matrix(double, num_fitpar, 1); chk_matrix(one_da);
    atry = new_vector(double, num_fitpar); chk_vector(atry);
    da = new_vector(double, num_fitpar); chk_vector(da);
    beta = new_vector(double, num_fitpar); chk_vector(beta);
    fitpar = new_vector(double, num_fitpar); chk_vector(fitpar);
    alloc_csd( csd->num_sym_op, csd->n_atom, csd->num_lofg, &csd_try );
    alloc_csd_sub_cstr( csd->n_atom, csd->num_lofg, csd->lof_type, &csd_try );
    fed_try = new_vector(FIT_EXP_DATASET, num_exp_dataset); chk_vector(fed_try);
    for(i=0; i<num_exp_dataset; i++){
      alloc_ed(
        fed[i].beam_search, fed[i].num_exact + fed[i].num_bethe, fed[i].num_rg,
        fed[i].num_thickness, fei[i].num_data_points, &(fed_try[i])
        );
    }

    mb_fprintf(fo_log, "$$First calc.\n");
    mb_fprintf(stdout, "$$First calc.\n");
    *alamda = 0.001;
    marquardt_coef(
      csd, num_exp_dataset, fed, fei, num_fitpar, num_perturb,
      alpha, beta, chisq, fo_log
      );
    old_chisq = (*chisq);
    /* Intensities output */
    sprintf(int_outfile, "%s_fst.txt", infile);
    mb_fprintf(fo_log, "1st cycle intensity ==> %s\n", int_outfile);
    fit_intensity_output( num_exp_dataset, fed, fei, int_outfile );

    mb_fprintf(
      fo_log, "$$ chisq of the 1st calc. is %lf.\n", *chisq
      );
  }
  /* Simulation with no fitting */
  if( epsilon.no_fitting_flag == 1 ){
    return; 
  }

  /* Make covar & one_da */
  for (j=0; j<num_fitpar; j++) {
    for (k=0; k<num_fitpar; k++){ covar[j][k] = alpha[j][k]; }
    covar[j][j] = alpha[j][j] * ( 1.0 + (*alamda) );
    one_da[j][0] = beta[j];
  }
  if(dbg.covar_dump_flag == 1){  /* debug */
    fprintf(stdout, "$$ cov1:\n");
    print_doublemat(num_fitpar, covar);
  }

  /* ----- Solve eq. ----- */
  if( epsilon.svd_flag == 1 ){ calc_svd(covar, num_fitpar, one_da); }
  else{ gaussj(covar, num_fitpar, one_da, 1); }
  for (j=0; j<num_fitpar; j++) { da[j] = one_da[j][0]; }

  if(dbg.covar_dump_flag == 1){/* debug */
    fprintf(stdout, "$$ cov2:\n");
    print_doublemat(num_fitpar, covar);
  }

  if (*alamda == 0.0) { /* Termination */
    free_vector(beta); free_vector(da);
    free_vector(atry); free_matrix(one_da);
    free_vector(fitpar);
    free_csd(&csd_try);
    free_fed_vector(num_exp_dataset, fed_try);
    return;
  }

  /* Trial */
  mb_fprintf(fo_log, "$$ Trial vector:\n");
  for (j=0; j<num_fitpar; j++) {
    mb_fprintf(fo_log, "da[%d]=%lf;", j, da[j]);
  }
  mb_fprintf(fo_log, "\n");
  csd_fed_to_fitpar(csd, num_exp_dataset, fed, num_fitpar, fitpar);
  for (j=0; j<num_fitpar; j++) {
    atry[j] = fitpar[j] + da[j]; /* Trial parameters */
  }
  cpy_csd( csd, &csd_try ); cpy_fed( num_exp_dataset, fed, fed_try );
  fitpar_to_csd_fed( &csd_try, num_exp_dataset, fed_try, num_fitpar, atry );

  marquardt_coef(
    &csd_try, num_exp_dataset, fed_try, fei, num_fitpar,
    num_perturb, covar, da, chisq, fo_log
    );/* Note: use of covar & da instead of alpha & beta */

  if (*chisq < old_chisq) { /* Succeeded */
    *alamda *= 0.1;
    old_chisq = (*chisq);
    for (j=0; j<num_fitpar; j++) {
      for (k=0; k<num_fitpar; k++) {alpha[j][k] = covar[j][k];}
      beta[j] = da[j];
      fitpar[j] = atry[j];
    }
    cpy_fed( num_exp_dataset, fed_try, fed );
    fitpar_to_csd_fed(csd, num_exp_dataset, fed, num_fitpar, fitpar);
    *sf_flag = 1;
  }
  else {/* Failed */
    *alamda *= 10.0;
    *chisq = old_chisq;
    fitpar_to_csd_fed(csd, num_exp_dataset, fed, num_fitpar, fitpar);
    *sf_flag = 0;
  }
  if(*sf_flag == 1){
    mb_fprintf(fo_log, "$$ Trial succeeded.\n");
    mb_fprintf(stdout, "$$ Trial succeeded.\n");
  }
  else{
    mb_fprintf(fo_log, "$$ Trial failed.\n");
    mb_fprintf(stdout, "$$ Trial failed.\n");
  }
  return;
}


void converge_chk(
  FILE *fo_log, double chisq, double old_chisq, int num_fitpar,
  int sf_flag, double lambda, int *convergence_flag
  )
{
  if (chisq < epsilon.tol_chisq_abs * num_fitpar && sf_flag == 1){
    mb_fprintf( fo_log, "$$ Caution:\n" );
    mb_fprintf(
      fo_log,
      "$$ Chi square %lf is less than (tol_chisq_abs. * num_fitpar) %lf.\n",
      chisq, epsilon.tol_chisq_abs * num_fitpar
      );
    mb_fprintf( fo_log, "$$ Check your data again.\n");
    *convergence_flag = 1;
  }
  else if (
    old_chisq >= chisq && chisq > (1 - epsilon.tol_chisq_rel) * old_chisq
    && sf_flag == 1
    ){
    mb_fprintf( fo_log, "\n ***** Normal convergence. *****\n");
    mb_fprintf( stdout, "\n ***** Normal convergence. *****\n");
    *convergence_flag = 1;
  }
  /*
  else if (
    (1 + epsilon.tol_chisq_rel) * old_chisq > chisq && chisq > old_chisq
    ){
    mb_fprintf(fo_log, "Residual sum of squares has slightly increased.\n");
    mb_fprintf(
      fo_log, "This is probably due to a round-off error and tolerable.\n"
      );
    *convergence_flag = 1;
  }
   */
  else{
    *convergence_flag = 0; /* Not converged */
  }
}


void no_fit_output(
  FILE *fo_log, int num_exp_dataset, FIT_EXP_DATASET *fed,
  FIT_EXP_INTSET *fei, char *infile
  )
{
  mb_fprintf( fo_log, "\n***** Result:\n");
  print_scaled_r_factors( fo_log, num_exp_dataset, fed, fei );
  print_r_factors( fo_log, num_exp_dataset, fed, fei );
}


void fit_end_message(FILE *fo_log)
{
  mb_fprintf(fo_log, "\n +--------------------------------");
  mb_fprintf(fo_log, "------------------------------+\n");
  mb_fprintf(fo_log, " |MBFIT has done a fitting job. ");
  mb_fprintf(fo_log, "You should judge if it is right!|\n");
  mb_fprintf(fo_log, " +--------------------------------");
  mb_fprintf(fo_log, "------------------------------+\n\n");

  mb_fprintf(stdout, "\n +--------------------------------");
  mb_fprintf(stdout, "------------------------------+\n");
  mb_fprintf(stdout, " |MBFIT has done a fitting job. ");
  mb_fprintf(stdout, "You should judge if it is right!|\n");
  mb_fprintf(stdout, " +--------------------------------");
  mb_fprintf(stdout, "------------------------------+\n\n");
}


void output_corr_coef(
  CRYSTAL_STRCT_DATA *csd,  int num_exp_dataset, FIT_EXP_DATASET *fed,
  int num_fitpar, double **covar, char *outfile, FILE *fo_log
  )
{
  /* Output correlation_coef */
  int  i, j;
  double  corr, corr_eps, corr_thres;
  FIT_CTRL  fit;
  FILE  *fo;

  fo = mb_fopen_chk_wmode(outfile);
  mb_fprintf(fo, "%% %s\n", outfile);
  mb_fprintf(fo_log, "Correlation coef. output ==> %s\n", outfile);
  mb_fprintf(
    fo_log, "Correlation coef. (>%lf):\n", epsilon.corr_threshold
    );
  mb_fprintf(fo, "Correlation coefficients:\n");

  for(j=0; j<num_fitpar; j++) {
    fit = search_csd_fed(j, csd, num_exp_dataset, fed);
    mb_fprintf(fo, " par[%d]: %s\n", j, fit.name);
  }

  corr_eps = epsilon.eps;
  corr_thres = epsilon.corr_threshold;
  for(i=0; i<num_fitpar; i++){
    for(j=i+1; j<num_fitpar; j++){
      if ( covar[i][i] > corr_eps && covar[j][j] > corr_eps ){
        corr = covar[i][j] / (sqrt(covar[i][i]) * sqrt(covar[j][j]) );
        if( fabs(corr) > corr_thres){
          mb_fprintf (fo_log, " rho[%2d][%2d] = %5.2lf\n", i, j, corr );
        }
        mb_fprintf(fo, " rho[%2d][%2d] = %5.2lf\n", i, j, corr );
      }
    }
  }
  mb_fclose(fo);
}


double eval_disksize_ed(
  double H_begin, double H_end, double G_begin, double G_end
  )
{
  double disksize;
  disksize = 0.0;
  if(fabs(H_begin) > disksize){ disksize = fabs(H_begin); }
  if(fabs(H_end) > disksize){ disksize = fabs(H_end); }
  if(fabs(G_begin) > disksize){ disksize = fabs(G_begin); }
  if(fabs(G_end) > disksize){ disksize = fabs(G_end); }
  return disksize;
}


void input_fit_exp_dataset(
  FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei, FILE *fi
  )
{
  /*
    Input fit_exp_dataset from fi
     for an element of fit_exp_dataset
   */
  int  it, nt, ig, ng, ik, nd, tmp_nt, tmp_ng, nch, nx;
  HKL  tmp_g;
  char  buf[BUFLEN], message[BUFLEN];
  double  t[4], t_weight;
  FILE  *fint;

  /* INCIDENT_BEAM block */
  chk_data("INCIDENT_BEAM{", fi);
  ed->incidence = fget_uvw(fi);
  ed->surf_norm = fget_uvw(fi);
  ed->zolzbase1 = fget_r_hkl(fi);
  fget_fitpar(&(ed->H_tilt_f), &(ed->H_tilt), fi);
  fget_fitpar(&(ed->G_tilt_f), &(ed->G_tilt), fi);
  fget_fitpar(&(ed->accel_kvol_f), &(ed->accel_kvol), fi);
  chk_data("}INCIDENT_BEAM", fi);

  /* CALC_KXY_RANGE block */
  /*chk_data("CALC_KXY_RANGE{", fi);*/
  sprintf(
    message, "$$$$$ Caution!!! CALC_KXY_RANGE must be before BEAM_SEARCH. $$$$$"
    );
  chk_data_info("CALC_KXY_RANGE{", message, fi);
  ed->calc_range_flag = fget_i(fi);
  ed->calc_type = fget_i(fi);
  if(
    ed->calc_type == 0 || ed->calc_type == 1 ||
    ed->calc_type == 2 || ed->calc_type == 3
    ){
    ed->calc_numh = fget_i(fi);
    fget_2d( &(ed->calc_hbgn), &(ed->calc_hend), fi );
    fget_2d( &(ed->calc_gbgn), &(ed->calc_gend), fi );
  }
  else{
    mb_fprintf(stdout, "$$ Invalid calc_type=%d. Abort.\n", ed->calc_type);
    exit(EXIT_FAILURE);
  }
  /* printf(
    "$$ ed->calc_hbgn, ed->calc_hend, ed->calc_gbgn, ed->calc_gend: %f %f %f %f",
    ed->calc_hbgn, ed->calc_hend, ed->calc_gbgn, ed->calc_gend
    ); */
  ed->beam_search.disksize
    = eval_disksize_ed(
      ed->calc_hbgn, ed->calc_hend, ed->calc_gbgn, ed->calc_gend );
  mb_fprintf(stdout, "$$disksize = %f\n", ed->beam_search.disksize);
  chk_data("}CALC_KXY_RANGE", fi);

  /* BEAM_SEARCH block */
  chk_data("BEAM_SEARCH{", fi);
  fget_ed_beams(ed, fi);
  chk_data("}BEAM_SEARCH", fi);

  /* INTENSITY_DATA block */
  chk_data("INTENSITY_DATA{", fi);
  fget_1s( ed->intensity_file, fi );
  fint = fopen_chk_rmode(ed->intensity_file);

  /* Number of thicknesses */
  nt = ed->num_thickness = ei->num_thickness = fget_i(fi);
  tmp_nt = fget_i(fint);
  mb_fprintf(stdout, "$$ nt, tmp_nt = %d, %d\n", nt, tmp_nt);
  if( nt != tmp_nt ){
    mb_fprintf(
      stdout,
      "$$ num_thickness = %d is not equal to int-file's = %d. Abort.\n",
      nt, tmp_nt
      );
    exit(EXIT_FAILURE);
  }

  /* Memory allocation */
  ed->scale_factor = new_vector(double, nt);
  chk_vector(ed->scale_factor);
  ed->scale_factor_f = new_vector(FIT_CTRL, nt);
  chk_vector(ed->scale_factor_f);
  ed->thickness = new_vector(double, nt);
  chk_vector(ed->thickness);
  ed->thickness_f = new_vector(FIT_CTRL, nt);
  chk_vector(ed->thickness_f);
  ed->thickwidth = new_vector(double, nt);
  chk_vector(ed->thickwidth);
  ed->thickwidth_f = new_vector(FIT_CTRL, nt);
  chk_vector(ed->thickwidth_f);

  ed->num_avrpix = new_vector(int, nt);
  chk_vector(ed->num_avrpix);
  ed->gaussian_par = new_vector(double, nt);
  chk_vector(ed->gaussian_par);
  ed->avrpix_wt = (double ***) new_tensor_element(nt);
  chk_tensor(ed->avrpix_wt);

  /* ng: number of reflections of experimental data */
  ng = ed->num_rg = ei->num_rg = fget_i(fi);
  tmp_ng = fget_i(fint);
  mb_fprintf(stdout, "$$ ng, tmp_ng = %d, %d\n", ng, tmp_ng);
  if( ng != tmp_ng ){
    mb_fprintf(
      stdout,
      "$$ Num. of reflex. = %d is not equal to the int file's = %d. Abort.\n",
      ng, tmp_ng
      );
    exit(EXIT_FAILURE);
  }
  chk_data("}INTENSITY_DATA", fi);

  /* Memory allocations */
  ei->obs_rg = new_vector(HKL, ng);        chk_vector(ei->obs_rg);
  ed->g = new_vector(double, ng);          chk_vector(ed->g);
  ed->art_weight = new_vector(double, ng); chk_vector(ed->art_weight);

  ed->H_shift = new_matrix(double, ng, nt);      chk_matrix(ed->H_shift);
  ed->G_shift = new_matrix(double, ng, nt);      chk_matrix(ed->G_shift);
  ed->H_shift_f = new_matrix(FIT_CTRL, ng, nt);  chk_matrix(ed->H_shift_f);
  ed->G_shift_f = new_matrix(FIT_CTRL, ng, nt);  chk_matrix(ed->G_shift_f);

  ei->num_data_points = new_vector(int, ng);
  chk_vector(ei->num_data_points);

  ei->obs_H = (double ***) new_tensor_element(ng);  chk_tensor(ei->obs_H);
  ei->obs_G = (double ***) new_tensor_element(ng);  chk_tensor(ei->obs_G);
  ei->obs_intensity = (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_intensity);
  ei->cal_intensity = (double ***) new_tensor_element(ng);
  chk_tensor(ei->cal_intensity);
  ei->obs_sig = (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_sig);
  for(ig=0; ig<ng; ig++){
    ei->obs_H[ig] = (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_H[ig]);
    ei->obs_G[ig] = (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_G[ig]);
    ei->obs_intensity[ig] = (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_intensity[ig]);
    ei->cal_intensity[ig] = (double **) new_matrix_element(nt);
    chk_matrix(ei->cal_intensity[ig]);
    ei->obs_sig[ig] = (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_sig[ig]);
  }

  for(it=0; it<nt; it++){
    /* THICKNESS_BLOCK part */
    chk_data("THICKNESS_BLOCK{", fi);

    /* STW block */
    chk_data("STW_PAR{", fi);
    fget_fitpar(&(ed->scale_factor_f[it]), &(ed->scale_factor[it]), fi);
    fget_fitpar(&(ed->thickness_f[it]), &(ed->thickness[it]), fi);
    fget_fitpar(&(ed->thickwidth_f[it]), &(ed->thickwidth[it]), fi);
    if(ed->thickwidth[it] > epsilon.eps && epsilon.num_thickwidth == 0){
      mb_fprintf(
        stdout, "$$ thickwidth = %lf and num_thickwidth = %d. Abort.\n",
        ed->thickwidth[it], epsilon.num_thickwidth
        );
      exit(EXIT_FAILURE);
    }
    fget_2d(&(t[0]), &(t[1]), fi);  /* Gaussian convolution of pixels */
    nx = ed->num_avrpix[it] = (int) t[0];
    ed->gaussian_par[it] = t[1];
    if(nx > 0){
      ed->avrpix_wt[it] = new_matrix(double, nx, nx);
      chk_matrix(ed->avrpix_wt[it]);
    }
    else{
      ed->avrpix_wt[it] = new_matrix(double, 1, 1);	/* <<<<< dummy 2001.08.22 */
      chk_matrix(ed->avrpix_wt[it]);
    }
    calc_avrpix_wt(
      ed->num_avrpix[it], ed->avrpix_wt[it], ed->gaussian_par[it]
      );
    chk_data("}STW_PAR", fi);

    /* REFLEX_SHIFT_PAR block */
    chk_data("REFLEX_SHIFT_PAR{", fi);
    for(ig=0; ig<ng; ig++){
      ei->obs_rg[ig] = fget_hkl(fint);  /* from .int file */
      fget_hkl_weight( &tmp_g, &(ed->art_weight[ig]), fi );
      /*ed->art_weight[ig] = t_weight;*/
        /*
      comment_skip_fgets(sizeof buf, buf, fi);
      nch = str_to_fs(4, buf, t);
      if (nch == 3) {
        tmp_g.h = (int) t[0];
        tmp_g.k = (int) t[1];
        tmp_g.l = (int) t[2];
        ed->art_weight[ig] = 1.0;
      }
      else if (nch == 4) {
        tmp_g.h = (int) t[0];
        tmp_g.k = (int) t[1];
        tmp_g.l = (int) t[2];
        ed->art_weight[ig] = t[3];
      }
      else {
        mb_fprintf(stdout, "$$ Data error in hkl and weight. Abort.");
        exit(EXIT_FAILURE);
      }
         */
      if(
        ei->obs_rg[ig].h != tmp_g.h
        || ei->obs_rg[ig].k != tmp_g.k 
        || ei->obs_rg[ig].l != tmp_g.l
        ){
        mb_fprintf(stdout, "$$ hkl is inconsistent: \n");
        mb_fprintf(
          stdout, "$$ par:%d %d %d : int:%d %d %d.    Abort.\n",
          tmp_g.h, tmp_g.k, tmp_g.l,
          ei->obs_rg[ig].h, ei->obs_rg[ig].k, ei->obs_rg[ig].l
          );
        exit(EXIT_FAILURE);
      }
      fget_fitpar(&(ed->H_shift_f[ig][it]), &(ed->H_shift[ig][it]), fi);
      fget_fitpar(&(ed->G_shift_f[ig][it]), &(ed->G_shift[ig][it]), fi);
      nd = ei->num_data_points[ig] = fget_i(fint);
      /* mb_fprintf(stdout, "&& .fint nd=%d\n", nd); */
      
      ei->obs_H[ig][it] = new_vector_element(double, nd);
      chk_vector(ei->obs_H[ig][it]);
      ei->obs_G[ig][it] = new_vector_element(double, nd);
      chk_vector(ei->obs_G[ig][it]);
      ei->obs_intensity[ig][it] = new_vector_element(double, nd);
      chk_vector(ei->obs_intensity[ig][it]);
      ei->cal_intensity[ig][it] = new_vector_element(double, nd);
      chk_vector(ei->cal_intensity[ig][it]);
      ei->obs_sig[ig][it] = new_vector_element(double, nd);
      chk_vector(ei->obs_sig[ig][it]);

      for(ik=0; ik<nd; ik++){
        fget_4d(
          &(ei->obs_H[ig][it][ik]), &(ei->obs_G[ig][it][ik]),
          &(ei->obs_intensity[ig][it][ik]), &(ei->obs_sig[ig][it][ik]),
          fint
          );
        if(ei->obs_sig[ig][it][ik] <= 0.0){	/* <<<<< */
          mb_fprintf(stdout, "$$ warning !!: sig <= 0.0\n");
        }
      }
      chk_data("RF_END", fint);
    }
    chk_data("}REFLEX_SHIFT_PAR", fi);
    chk_data("}THICKNESS_BLOCK", fi);
    chk_data("THICK_END", fint);
  }
  chk_data("INTENSITY_DATA_END", fint);
  fclose(fint);
}


void mb_fitting( FILE *fi, FILE *fo_log, char *infile )
{
  /*
   * Non-linear least square fitting
   *  (fo_log: file pointer for log output)
   */
  CRYSTAL_STRCT_DATA csd;
  FIT_EXP_DATASET *fed;
  FIT_EXP_INTSET  *fei;
  double  **covar, **alpha, chisq, old_chisq, lambda;
  int  num_exp_dataset, num_fitpar, num_par, num_perturb;
  int  maxnum_iter, convergence_flag, sf_flag, ie, iter;
  char  log_outfile[BUFLEN], int_outfile[BUFLEN];
  char  out_parfile[BUFLEN], out_corr_file[BUFLEN];

  dbg_dump_ini_fit();

  input_csd_cstr(&csd, fi, infile);   /* crystal structure data */

  /* Experimental data */
  chk_data("FIT_CALCULATION_CONDITION{", fi);
  chk_data("ITER_EXP_NUM{", fi);
  maxnum_iter = fget_i(fi); /* maximum number of iteration */
  if (maxnum_iter <= 1){ epsilon.no_fitting_flag = 1; }
  num_exp_dataset = fget_i(fi);
  mb_fprintf(
    fo_log, "Number of experimental data set = %d\n", num_exp_dataset
    );
  chk_data("}ITER_EXP_NUM", fi);
  fed = new_vector(FIT_EXP_DATASET, num_exp_dataset);
  chk_vector(fed);
  fei = new_vector(FIT_EXP_INTSET,  num_exp_dataset);
  chk_vector(fei);
  for(ie=0; ie<num_exp_dataset; ie++){
    chk_data("DATA_SET{", fi);
    input_fit_exp_dataset( &(fed[ie]), &(fei[ie]), fi );
    chk_data("}DATA_SET", fi);
  }
  chk_data("}FIT_CALCULATION_CONDITION", fi);
  fclose(fi);
  constraint_fed(num_exp_dataset, fed);

  mb_fprintf(fo_log, "%% Fitting log\n");
  print_int_ratio( fo_log, num_exp_dataset, fed, fei );

  if(
    dbg.eigval_dump_flag == 1 || dbg.eigvec_dump_flag == 1
    || dbg.vg_dump_flag == 1  || dbg.a_mat_dump_flag == 1
    || dbg.b_mat_dump_flag == 1 || dbg.b_cond_dump_flag == 1
    ){
    epsilon.no_fitting_flag = 1;
  }

  if( epsilon.no_fitting_flag == 1 ){
    maxnum_iter = 1;
  }

  mb_fprintf(fo_log, "Iteration limit = %d times.\n", maxnum_iter);
  mb_fprintf(stdout, "Iteration limit = %d times.\n", maxnum_iter);
  mb_fprintf(fo_log, "no_fitting_flag = %d.\n", epsilon.no_fitting_flag);
  mb_fprintf(stdout, "no_fitting_flag = %d.\n", epsilon.no_fitting_flag);

  count_num_fitpar(
    &csd, num_exp_dataset, fed, &num_fitpar, &num_par, &num_perturb
    );
  /* if (maxnum_iter == 1){ num_fitpar = 0; } */
  mb_fprintf(fo_log, "$$ num_fitpar  = %d\n", num_fitpar);
  mb_fprintf(stdout, "$$ num_fitpar  = %d\n", num_fitpar);
  mb_fprintf(fo_log, "$$ num_par     = %d\n", num_par);
  mb_fprintf(stdout, "$$ num_par     = %d\n", num_par);
  mb_fprintf(fo_log, "$$ num_perturb = %d\n", num_perturb);
  mb_fprintf(stdout, "$$ num_perturb = %d\n", num_perturb);
  set_csd_fed_step( &csd, num_exp_dataset, fed ); 

  mb_fprintf(fo_log, "\n***** Initial state\n" );
  par_output_short( &csd, num_exp_dataset, fed, fo_log );

  if( num_fitpar > 0 ){
    alpha = new_matrix(double, num_fitpar, num_fitpar);  chk_matrix(alpha);
    covar = new_matrix(double, num_fitpar, num_fitpar);  chk_matrix(covar);
  }

  iter = 0;
  convergence_flag = 0;
  old_chisq = 1e+18; /* <<<<< */
  lambda = -1.0; /* For initialization of marquardt_min */

  /* Fitting loop */
  do{
    iter++;
    if (iter > maxnum_iter) {
      mb_fprintf(fo_log, "$$ iter > maxnum_iter.\n");
      mb_fprintf(stdout, "$$ iter > maxnum_iter.\n");
      break;
    }
    mb_fprintf(fo_log, "\n***** Iter = %d\n", iter);
    mb_fprintf(stdout, "\n***** Iter = %d\n", iter);

    marquardt_min(
      &csd, num_exp_dataset, fed, fei, infile, maxnum_iter, num_fitpar,
      num_perturb, covar, alpha, &chisq, &lambda, &sf_flag, fo_log
      );/* Levenberg-Marquardt method */

    sprintf(int_outfile, "%s_mid.txt", infile);
    mb_fprintf(fo_log, "Intensity output ==> %s\n", int_outfile);
    fit_intensity_output( num_exp_dataset, fed, fei, int_outfile );

    sprintf(out_parfile, "%s_rs.txt", infile);
    output_result_parfile(
      out_parfile, &csd, maxnum_iter, num_exp_dataset, fed, fei
      );

    if(epsilon.no_fitting_flag == 0){/* For fitting */

      /*mb_fprintf(fo_log, "Chi_sq = %lg\n", chisq); */
      print_scaled_r_factors( fo_log, num_exp_dataset, fed, fei );
      print_r_factors( fo_log, num_exp_dataset, fed, fei );
      par_output_short( &csd, num_exp_dataset, fed, fo_log );
      mb_fprintf(
        fo_log, "$$ chisq=%lg, old_chisq=%lg, sf=%d, lambda=%lg\n",
        chisq, old_chisq, sf_flag, lambda
        );
      mb_fprintf(
        stdout, "$$ chisq=%lg, old_chisq=%lg, sf=%d, lambda=%lg\n",
        chisq, old_chisq, sf_flag, lambda
        );
      converge_chk(
        fo_log, chisq, old_chisq, num_fitpar, sf_flag, lambda,
        &convergence_flag
        );
    }
    old_chisq = chisq;
  } while( convergence_flag != 1);

  if( epsilon.no_fitting_flag == 0 ){/* For fitting */
    lambda = 0.0;	/* For calculation of alpha and covar. matrices */
    marquardt_min(
      &csd, num_exp_dataset, fed, fei, infile,
      maxnum_iter, num_fitpar, num_perturb,
      covar, alpha, &chisq, &lambda, &sf_flag, fo_log
      );

    sprintf(int_outfile, "%s_fnl.txt", infile);
    mb_fprintf(
      fo_log, "Final result intensities ==> %s\n", int_outfile
      );
    fit_intensity_output( num_exp_dataset, fed, fei, int_outfile );

    sprintf(int_outfile, "%s_mid.txt", infile);
    mb_fprintf(fo_log, "Intensity output ==> %s\n", int_outfile);
    fit_intensity_output( num_exp_dataset, fed, fei, int_outfile );

    fitpar_sigma_to_csd_fed( &csd, num_exp_dataset, fed, num_fitpar, covar);
    sprintf(out_corr_file, "%s_cor.txt", infile);
    output_corr_coef(
      &csd, num_exp_dataset, fed, num_fitpar, covar, out_corr_file, fo_log
      );
    mb_fprintf(fo_log, "\n***** Result\n");
    mb_fprintf(stdout, "\n***** Result\n");
    mb_fprintf(fo_log, "Iteration = %d\n", iter);
    mb_fprintf(stdout, "Iteration = %d\n", iter);
    mb_fprintf(fo_log, "Chi_sq = %lg\n", chisq);
    mb_fprintf(stdout, "Chi_sq = %lg\n", chisq);
    print_scaled_r_factors( fo_log, num_exp_dataset, fed, fei );
    print_r_factors( fo_log, num_exp_dataset, fed, fei );
    par_output_short( &csd, num_exp_dataset, fed, fo_log );

    sprintf(out_parfile, "%s_rs.txt", infile);
    output_result_parfile(
      out_parfile, &csd, maxnum_iter, num_exp_dataset, fed, fei
      );
  }
  else{	/* no_fitting */
    no_fit_output( fo_log, num_exp_dataset, fed, fei, infile );
  }

  fit_end_message(fo_log);

  free_matrix(alpha);
  free_matrix(covar);
  free_fed_vector(num_exp_dataset, fed);
  free_fei_vector(num_exp_dataset, fei);
  free_csd(&csd);
}
