/*
 * ts_scan.c
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "mbconfig.h"
#include "fitting.h"
#include "utils.h"
#include "mb_input.h"
#include "data_out.h"
#include "mb_sim.h"
#include "svd.h"
#include "mb_mpi.h"

extern DEBUG_INFO dbg;
extern EPS_CTRL	epsilon;

#define BUFLEN  256
/*#define PARNAME 20 */

/*-----*/

double cal_weight(
  double obs_sig, double obs_intensity, double cal_intensity, double g,
  double art_weight
  );
double cal_weight_factor(
  double obs_sig, double obs_intensity, double cal_intensity,
  double artificial_weight
  );
void calc_K_xy_cbed(
  int *num_k, int *num_k_cal,
  R_HKL k_xy[], HGPIX k_pix[], double H_tilt, double G_tilt,
  UVW incidence, R_HKL zolzbase1, CELL_PAR bv, int calc_type,
  double G_bgn, double G_end, double H_bgn, double H_end, int num_points
  );
void interpolation_2d(
  double *itp_intensity, double *intensity, R_HKL *k_xy, HGPIX *k_pix,
  double H_shift, double G_shift, double *obs_H, double *obs_G,
  int obs_num_points, int npixels, int nk,
  int num_avrpix, double **avrpix_wt
  );


/* ----- TS_SCAN ------------------------------------------------------- */
/* --------------------------------------------------------------------- */
void norm_cross_correlation(
  int org_image_width, int org_image_height, double **org_image,
  int tmpl_image_width, int tmpl_image_height, double **tmpl_image,
  double **cross_corr_image
  )
{
  /*
    Normalized cross correlation coefficient:
     Image_width and _heigth of cross_corr_image[][] are the same as
     those of org_image[][].
 */

  int i, j, m, n;
  double tmpu, tmpd1, tmpd2, val_org, val_tmpl;

  /* Check image sizes. */
  if(org_image_width < tmpl_image_width){
    mb_fprintf(stdout, "$$ org_image_width = %d should be greater than\n");
    mb_fprintf(stdout, 
      "$$  tmpl_image_width=%d. Abort.\n", org_image_width, tmpl_image_width
      );
    exit(EXIT_FAILURE);
  }
  if(org_image_height < tmpl_image_height){
    mb_fprintf(stdout, "$$ org_image_height = %d should be greater than\n");
    mb_fprintf(stdout, 
      "$$ tmpl_image_heigth=%d. Abort.\n", org_image_height, tmpl_image_height
      );
    exit(EXIT_FAILURE);
  }

  /* Initialize */
  for(m=0; m<org_image_width; m++){
    for(n=0; n<org_image_height; n++){
      cross_corr_image[m][n] = 0.0;
    }
  }

  tmpu = 0.0; tmpd1 = 0.0; tmpd2 = 0.0;
  for(m=0; m<org_image_width-tmpl_image_width; m++){
    for(n=0; n<org_image_height-tmpl_image_height; n++){
      /* Cross correlation */
      for(i=m; i<tmpl_image_width+m; i++){
        for(j=n; j<tmpl_image_height+n; j++){
          val_org = org_image[i][j];
          val_tmpl = tmpl_image[i-m][j-n];
          tmpu  += val_org * val_tmpl;
          tmpd1 += sq(val_org);
          tmpd2 += sq(val_tmpl);
        }
      }
      cross_corr_image[m][n] = tmpu / ( sqrt(tmpd1) * sqrt(tmpd2) );
    }
  }
}


double scale_factor_disk(
  int num_points, double *cal_int, double *obs_sig, double *obs_int
  )
{
  /* Scale factor for one disk */
  int  ik;
  double  su, sd, c_int, o_sig, o_int, weight, scale, art_wt=1.0;

  su = 0.0;
  sd = 0.0;
  for(ik=0; ik<num_points; ik++){
    c_int = cal_int[ik];
    o_sig = obs_sig[ik];
    o_int = obs_int[ik];
    weight = cal_weight_factor(o_sig, o_int, c_int, art_wt);
    /*
    printf(
      "$$ik, c_int, o_sig, o_int, sig2i= %d %lf %lf %lf %lf\n",
      ik, c_int, o_sig, o_int, sig2i
      );
      */
    su += o_int * c_int * weight;
    sd += sq(c_int) * weight;
  }

  if(sd > epsilon.eps){
    scale = su / sd;
    /*printf("$$ scale = %lf\n", scale);*/
  }
  else{
    mb_fprintf(stdout, 
      "$$ (scale_factor_disk) Too small denominator (%lf).\n", sd
      );
    mb_fprintf(stdout, "$$ scale is set to be 1.0.\n");
    scale = 1.0;
  }
  return scale;
}


double chi_square_disk(
  int num_points, double *cal_int, double *obs_sig, double *obs_int,
  double scale
  )
{
  /* Calc. of Chi-squares */
  int  ik;
  double  c_int, o_sig, o_int, weight, dy, chisq, art_wt=1.0;

  chisq = 0.0;
  for(ik=0; ik<num_points; ik++){
    c_int = cal_int[ik] * scale;
    o_sig = obs_sig[ik];
    o_int = obs_int[ik];
    weight = cal_weight_factor(o_sig, o_int, c_int, art_wt);
    dy = o_int - c_int;
    chisq += sq(dy) * weight;
    if(dbg.obs_cal_dump_flag == 1){ /* debug */
      mb_fprintf( stdout, "$$ obs, cal = %lf, %lf\n", o_int, c_int );
    }
  }
  return chisq;
}


void find_gof_min(
  double **gof_shiftscan, int num_shiftscan_x, int num_shiftscan_y,
  int *ix_at_minimum, int *iy_at_minimum, int *success_flag
  )
{
  int ix, iy, ix_min, iy_min;
  double min_gof;
  
  min_gof = 1e20;
  ix_min = iy_min = 0;
  for(ix=0; ix<num_shiftscan_x; ix++){
    for(iy=0; iy<num_shiftscan_y; iy++){
      if(min_gof > gof_shiftscan[ix][iy]){
        min_gof = gof_shiftscan[ix][iy];
        ix_min = ix;
        iy_min = iy;
      }
    }
  }
  
  if (ix_min != 0 && iy_min != 0){
    *success_flag = 1;  /* success */
  }
  else{
    *success_flag = 0;  /* fail */
  }
  
  *ix_at_minimum = ix_min;
  *iy_at_minimum = iy_min;
}


void read_hg_shift_weight_file(
  int num_reflex_preset, char *HGshift_wt_file,
  double *H_shift, double *G_shift, double *g_weight
  )
{
  int  ig, nch;
  double t[4];
  HKL  g;
  char buf[BUFLEN];
  FILE  *fi;
  
  fi = fopen_chk_rmode(HGshift_wt_file);
  chk_data("REFLEX_SHIFT_PAR{", fi);
  for(ig=0; ig<num_reflex_preset; ig++){
    comment_skip_fgets(sizeof buf, buf, fi);
    nch = str_to_fs(4, buf, t);
    if (nch == 3) {
      g.h = (int) t[0];  g.k = (int) t[1];  g.l = (int) t[2];
      g_weight[ig] = 1.0;
    }
    else if (nch == 4) {
      g.h = (int) t[0];  g.k = (int) t[1];  g.l = (int) t[2];
      g_weight[ig] = t[3];
    }
    else {
      mb_fprintf(stdout, "$$ Data error in hkl and weight. Abort.");
      exit(EXIT_FAILURE);
    }
    H_shift[ig] = fget_d(fi);
    G_shift[ig] = fget_d(fi);
  }
  chk_data("}REFLEX_SHIFT_PAR", fi);
  fclose(fi);
}


void read_hg_shift_file(
  int num_reflex_preset, char *HGshift_wt_file,
  double *H_shift, double *G_shift
  )
{
  /* g_weight will be ignored. */
  int  ig, nch;
  double t[4];
  char buf[BUFLEN];
  FILE  *fi;
  
  fi = fopen_chk_rmode(HGshift_wt_file);
  chk_data("REFLEX_SHIFT_PAR{", fi);
  for(ig=0; ig<num_reflex_preset; ig++){
    comment_skip_fgets(sizeof buf, buf, fi);
    nch = str_to_fs(4, buf, t);
    if (nch == 3 || nch == 4) {
      ;
    }
    else {
      mb_fprintf(stdout, "$$ Data error in hkl and weight. Abort.");
      exit(EXIT_FAILURE);
    }
    H_shift[ig] = fget_d(fi);
    G_shift[ig] = fget_d(fi);
  }
  chk_data("}REFLEX_SHIFT_PAR", fi);
  fclose(fi);
}


void read_int_file(FIT_EXP_INTSET *ei, char *intfile)
{
  int  nt, it, ng, ig, ik, nk;
  FILE  *fi;
  /*printf("$$(read_int_file) intfile=%s\n", intfile);*/
  fi = fopen_chk_rmode(intfile);
  
  nt = ei->num_thickness = fget_i(fi);
  ng = ei->num_rg = fget_i(fi);
  
  ei->obs_rg = new_vector(HKL, ng);  chk_vector(ei->obs_rg);
  ei->num_data_points = new_vector(int, ng);  chk_vector(ei->num_data_points);
  ei->obs_H = (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_H);
  ei->obs_G = (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_G);
  ei->obs_intensity	= (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_intensity);
  ei->cal_intensity	= (double ***) new_tensor_element(ng);
  chk_tensor(ei->cal_intensity);
  ei->obs_sig		= (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_sig);
  
  for(ig=0; ig<ng; ig++){
    ei->obs_H[ig]  = (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_H[ig]);
    ei->obs_G[ig]  = (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_G[ig]);
    ei->obs_intensity[ig]  = (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_intensity[ig]);
    ei->cal_intensity[ig]  = (double **) new_matrix_element(nt);
    chk_matrix(ei->cal_intensity[ig]);
    ei->obs_sig[ig]  = (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_sig[ig]);
  }

  for(it=0; it<nt; it++){
    for(ig=0; ig<ng; ig++){
      ei->obs_rg[ig] = fget_hkl(fi);
      nk = ei->num_data_points[ig] = fget_i(fi);
      ei->obs_H[ig][it]  = new_vector_element(double, nk);
      chk_vector(ei->obs_H[ig][it]);
      ei->obs_G[ig][it]  = new_vector_element(double, nk);
      chk_vector(ei->obs_G[ig][it]);
      ei->obs_intensity[ig][it] = new_vector_element(double, nk);
      chk_vector(ei->obs_intensity[ig][it]);
      ei->cal_intensity[ig][it] = new_vector_element(double, nk);
      chk_vector(ei->cal_intensity[ig][it]);
      ei->obs_sig[ig][it] = new_vector_element(double, nk);
      chk_vector(ei->obs_sig[ig][it]);

      for(ik=0; ik<nk; ik++){
        fget_4d(
          &(ei->obs_H[ig][it][ik]), &(ei->obs_G[ig][it][ik]),
          &(ei->obs_intensity[ig][it][ik]), &(ei->obs_sig[ig][it][ik]),
          fi
          );
        if(ei->obs_sig[ig][it][ik] <= 0.0){	/* <<<<< */
          mb_fprintf(stdout, "$$ warning !!: sig <= 0.0\n");
        }
      }
      chk_data("RF_END", fi);
    }
    chk_data("THICK_END", fi);
  }
  chk_data("INTENSITY_DATA_END", fi);
  fclose(fi);
}


double eval_gof(
  FIT_EXP_INTSET *ei, double scale, int num_reflex_preset, double *g_weight
  )
{
  /* Calc. of GOF and Chi-square */
  int  ig, ng, it, ik, nk, total_num_k_points;
  double  chisq, dy;
  double  cal_int, obs_int, obs_sig, weight;

  it = 0;
  total_num_k_points = 0;
  chisq = 0.0;
  ng = ei->num_rg;
  if (ng != num_reflex_preset){
    printf("$$ Disagreement between ei->num_rg (=%d)\n", ei->num_rg);
    printf("$$  and num_reflex_preset (=%d).\n", num_reflex_preset);
    exit(EXIT_FAILURE);
  }
  for(ig=0; ig<ng; ig++){
    nk = ei->num_data_points[ig];
    total_num_k_points += nk;
    for(ik=0; ik<nk; ik++){
      cal_int
        = ei->cal_intensity[ig][it][ik] * scale;
      obs_sig = ei->obs_sig[ig][it][ik];
      obs_int = ei->obs_intensity[ig][it][ik];
      weight = cal_weight_factor(obs_sig, obs_int, cal_int, g_weight[ig]);
      dy = obs_int - cal_int;
      /*printf("$$obs_int, cal_int= %lf %lf\n", obs_int, cal_int); */
      chisq += sq(dy) * weight;
    }
  }
  return sqrt(chisq / (double) total_num_k_points);
}


double eval_scale_factor(
  FIT_EXP_INTSET *ei, int num_reflex_preset, double *g_weight
 )
{
  /* Evaluation of the scale factor */
  int  ig, ng, it, ik, nk;
  double  su, sd, scale;
  double  cal_int, obs_int, obs_sig, weight, art_wt;

  su = 0.0; sd = 0.0;
  it = 0;
  art_wt = 1.0;
  ng = ei->num_rg;
  if (ng != num_reflex_preset){
    printf("$$ Disagreement between ei->num_rg (=%d)\n", ei->num_rg);
    printf("$$  and num_reflex_preset (=%d).\n", num_reflex_preset);
    exit(EXIT_FAILURE);
  }
  for(ig=0; ig<ng; ig++){
    nk = ei->num_data_points[ig];
    for(ik=0; ik<nk; ik++){
      cal_int = ei->cal_intensity[ig][it][ik];
      obs_sig = ei->obs_sig[ig][it][ik];
      obs_int = ei->obs_intensity[ig][it][ik];
      weight = cal_weight_factor(obs_sig, obs_int, cal_int, g_weight[ig]);
      su += obs_int * cal_int * weight;
      sd += sq(cal_int) * weight;
    }
  }
  if(sd > epsilon.eps){
    scale = su / sd;
  }
  else{
    mb_fprintf(
      stdout, "$$(eval_scale_factor) Too small denominator(=%lf). Abort.\n", sd
      );
    exit(EXIT_FAILURE);
  }
  return scale;
}


void set_calint_to_ei(
  FIT_EXP_INTSET *ei, SIM_RES *sr
  )
{
  double  cal_int, obs_int, obs_sig, sig2i, scale;
  double  H_shift, G_shift;
  int  ig, ng, it;
  int  nx_pixels, ny_pixels;
  int  num_avrpix = 0;  /* dummy */
  double  **avrpix_wt;  /* dummy */

  /* ==> Check ei and sr */
  ng = ei->num_rg;
  it = 0;  num_avrpix = 0;
  H_shift = 0.0;  G_shift = 0.0;
  nx_pixels = sr->hpix_max - sr->hpix_min + 1;
  ny_pixels = sr->gpix_max - sr->gpix_min + 1;
  for(ig=0; ig<ng; ig++){
    if(
      sr->cal_g[ig].h == ei->obs_rg[ig].h &&
      sr->cal_g[ig].k == ei->obs_rg[ig].k &&
      sr->cal_g[ig].l == ei->obs_rg[ig].l
      ){
      interpolate2d_of_1d_array(
        sr->intensity[ig], sr->num_k_points, sr->hpix, sr->gpix,
        nx_pixels, ny_pixels, sr->H, sr->G,
        ei->cal_intensity[ig][it], ei->num_data_points[ig],
        ei->obs_H[ig][it], ei->obs_G[ig][it],
        H_shift, G_shift,
        num_avrpix, avrpix_wt /* << dummy */
      );
    }
    else{
      printf("$$ Disagreement was found.\n");
      printf(
        "$$  sr->cal_g[%d] (=%d %d %d)\n", ig, 
        sr->cal_g[ig].h, sr->cal_g[ig].k, sr->cal_g[ig].l
        );
      printf(
        "$$  ei->obs_rg[%d] (=%d %d %d)\n", ig, 
        ei->obs_rg[ig].h, ei->obs_rg[ig].k, ei->obs_rg[ig].l
        );
      printf("$$ Abort.\n");
      exit(EXIT_FAILURE);
      }
  }
}


void set_calint_to_ei_shift(
  FIT_EXP_INTSET *ei, SIM_RES *sr,
  int num_reflex, double *H_shift, double *G_shift
  )
{
  double  cal_int, obs_int, obs_sig, sig2i, scale;
  int  ig, ng, it;
  int  nx_pixels, ny_pixels;
  int  num_avrpix = 0;  /* dummy */
  double  **avrpix_wt;  /* dummy */

  /* ==> Check ei and sr */
  ng = ei->num_rg;
  if(num_reflex != ng){
    printf("$$(set_calint_to_ei_shift) disagreement in num_reflex. Abort.\n");
    exit(EXIT_FAILURE);
  }
  it = 0;  num_avrpix = 0;
  nx_pixels = sr->hpix_max - sr->hpix_min + 1;
  ny_pixels = sr->gpix_max - sr->gpix_min + 1;
  for(ig=0; ig<ng; ig++){
    if(
      sr->cal_g[ig].h == ei->obs_rg[ig].h &&
      sr->cal_g[ig].k == ei->obs_rg[ig].k &&
      sr->cal_g[ig].l == ei->obs_rg[ig].l
      ){
      interpolate2d_of_1d_array(
        sr->intensity[ig], sr->num_k_points, sr->hpix, sr->gpix,
        nx_pixels, ny_pixels, sr->H, sr->G,
        ei->cal_intensity[ig][it], ei->num_data_points[ig],
        ei->obs_H[ig][it], ei->obs_G[ig][it],
        H_shift[ig], G_shift[ig],
        num_avrpix, avrpix_wt /* << dummy */
        );
    }
    else{
      printf("$$ Disagreement was found.\n");
      printf(
        "$$  sr->cal_g[%d] (=%d %d %d)\n", ig, 
        sr->cal_g[ig].h, sr->cal_g[ig].k, sr->cal_g[ig].l
        );
      printf(
        "$$  ei->obs_rg[%d] (=%d %d %d)\n", ig, 
        ei->obs_rg[ig].h, ei->obs_rg[ig].k, ei->obs_rg[ig].l
        );
      printf("$$ Abort.\n");
      exit(EXIT_FAILURE);
    }
  }
}


void read_sim_data(
  SIM_RES *sr, char *simfile, double Hshift_SIM, double Gshift_SIM
  )
{
  /*
    data read & malloc
   */
  int  ik, nk, ig, ng, ie;
  int  hpix_max, gpix_max, hpix_min, gpix_min;
  double  t[7], max_val, min_val, thickness;
  UVW  incidence;
  R_HKL  zolzbase1;
  HKL  g;
  char  buf[BUFLEN];
  CELL_PAR  bv;
  FILE *fp;
  
  fp = fopen_chk_rmode(simfile);
  chk_data("SIM_RESULT", fp);
  chk_data("VISUAL{", fp);
  fget_2d(&max_val, &min_val, fp);
  fget_d(fp);  /* gamma_coef dummy */
  fget_i(fp);  /* mag_coef dummy */
  fget_i(fp);  /* cont_rev_flag  dummy */
  chk_data("}VISUAL", fp);
  
  chk_data("CR_PARAMETERS{", fp);
  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(7, buf, t);
  bv.s11 = t[0]; bv.s22 = t[1]; bv.s33 = t[2];
  bv.s12 = t[3]; bv.s13 = t[4]; bv.s23 = t[5]; bv.vc = t[6];
  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(6, buf, t);
  bv.aas = t[0]; bv.bbs = t[1]; bv.ccs = t[2];
  bv.abs = t[3]; bv.bcs = t[4]; bv.acs = t[5];
  comment_skip_fgets(sizeof buf, buf, fp);
  str_to_fs(6, buf, t);
  bv.aa = t[0]; bv.bb = t[1]; bv.cc = t[2];
  bv.ab = t[3]; bv.bc = t[4]; bv.ac = t[5];
  sr->bv = bv;
  fget_3i( &(incidence.u), &(incidence.v), &(incidence.w), fp );
  sr->incidence = incidence;
  fget_3d( &(zolzbase1.h), &(zolzbase1.k), &(zolzbase1.l), fp );
  sr->zolzbase1 = zolzbase1;
  sr->thickness = thickness = fget_d(fp);
  chk_data("}CR_PARAMETERS", fp);

  chk_data("RF_INDEX{", fp);
  ng = sr->num_cal_g = fget_i(fp);  /* number of reflections */
  sr->cal_g = new_vector(HKL, ng);  chk_vector(sr->cal_g);
  sr->xpos = new_vector(double, ng);  chk_vector(sr->xpos);
  sr->ypos = new_vector(double, ng);  chk_vector(sr->ypos);
  for(ig=0; ig<ng; ig++){
    comment_skip_fgets(sizeof buf, buf, fp);  str_to_fs(5, buf, t);
    sr->cal_g[ig].h = (int) t[0];
    sr->cal_g[ig].k = (int) t[1];
    sr->cal_g[ig].l = (int) t[2];
    sr->xpos[ig] = t[3];
    sr->ypos[ig] = t[4];
  }
  chk_data("}RF_INDEX", fp);

  chk_data("COORD{", fp);
  nk = sr->num_k_points = fget_i(fp);
  fget_2d(&(sr->H_max), &(sr->H_min) ,fp);
  sr->H_max -= Hshift_SIM;  sr->H_min -= Hshift_SIM;  /* <<<<< */
  fget_2d(&(sr->G_max), &(sr->G_min) ,fp);
  sr->G_max -= Gshift_SIM;  sr->G_min -= Gshift_SIM;  /* <<<<< */
  fget_2i(&(sr->hpix_max), &(sr->hpix_min), fp);
  fget_2i(&(sr->gpix_max), &(sr->gpix_min), fp);
  hpix_max = sr->hpix_max;  hpix_min = sr->hpix_min;
  gpix_max = sr->gpix_max;  gpix_min = sr->gpix_min;
  sr->H = new_vector(double, nk);  chk_vector(sr->H);
  sr->G = new_vector(double, nk);  chk_vector(sr->G);
  sr->hpix = new_vector(int, nk);  chk_vector(sr->hpix);
  sr->gpix = new_vector(int, nk);  chk_vector(sr->gpix);
  for(ik=0; ik<nk; ik++){
    comment_skip_fgets(sizeof buf, buf, fp);  str_to_fs(4, buf, t);
    sr->H[ik] = t[0] - Hshift_SIM;  /* <<<<< */
    sr->G[ik] = t[1] - Gshift_SIM;  /* <<<<< */
    sr->hpix[ik] = (int) t[2] - hpix_min; /* <<<<< for interpolate2d */
    sr->gpix[ik] = (int) t[3] - gpix_min; /* <<<<< for interpolate2d */
    /*
    if(sr->hpix[ik] < 0 || sr->gpix[ik]){
      printf("$$ warning: sr->hpix[ik] or sr->gpix[ik] < 0.\n");
      exit(EXIT_FAILURE);
    }
     */
  }
  chk_data("}COORD", fp);

  sr->intensity = new_matrix(double, ng, nk);
  chk_vector(sr->intensity);
  for(ig=0; ig<ng; ig++){
    chk_data("RF{", fp);
    fget_3i(&(g.h), &(g.k), &(g.l), fp); /* dummy */
    for(ik=0; ik<nk; ik++){
      sr->intensity[ig][ik] = fget_d(fp);
    }
    chk_data("}RF", fp);
  }
}


void free_sr(SIM_RES *sr)
{
  if(sr->num_cal_g != 0){
    free_vector(sr->xpos);
    free_vector(sr->ypos);
    free_vector(sr->cal_g);
  }
  if(sr->num_k_points != 0){
    free_vector(sr->hpix);
    free_vector(sr->gpix);
    free_vector(sr->H);
    free_vector(sr->G);
  }
  if(sr->num_cal_g != 0 && sr->num_k_points != 0){
    free_matrix(sr->intensity);
  }
}


#if 0
void  gof_sim_int(
  char *simfile, char *intfile,
  double *thickness, double *gof, double *scalef, int *num_reflex
  )
{
  /* without HG scan */
  FIT_EXP_INTSET  ei;
  SIM_RES sr;
  
  read_int_file(&ei, intfile);
  read_sim_data(&sr, simfile);
  *thickness = sr.thickness;
  *num_reflex = sr.num_cal_g;
  
  set_calint_to_ei( &ei, &sr );
  *scalef = eval_scale_factor( &ei );
  *gof = eval_gof( &ei, *scalef );
  
  free_ei(&ei);
  free_sr(&sr);
}
#endif


void set_calint_to_ei_shiftscan(
  FIT_EXP_INTSET *ei, SIM_RES *sr,
  int num_shiftscan_x, double shiftscan_x_begin, double shiftscan_x_end,
  int num_shiftscan_y, double shiftscan_y_begin, double shiftscan_y_end,
  int num_g, double *Hshift_opt, double *Gshift_opt,
  double *Scale_opt, double *gof_reflex, int *success_flag
  )
{
  double  cal_int, obs_int, obs_sig, sig2i;
  int  ig, ng, it, ix, iy, min_ix, min_iy;
  int  nx_pixels, ny_pixels;
  int  num_avrpix = 0;  /* dummy */
  double  **avrpix_wt;  /* dummy */
  double  *H_shift, *G_shift;
  double  **scale, **gof_shiftscan;
  
  H_shift = new_vector(double, num_shiftscan_x);  chk_vector(H_shift);
  G_shift = new_vector(double, num_shiftscan_y);  chk_vector(G_shift);
  scale = new_matrix(double, num_shiftscan_x, num_shiftscan_y);
  chk_matrix(scale);
  gof_shiftscan = new_matrix(double, num_shiftscan_x, num_shiftscan_y);
  chk_matrix(gof_shiftscan);

  for(ix=0; ix<num_shiftscan_x; ix++){
    H_shift[ix] = shiftscan_x_begin +
      (shiftscan_x_end - shiftscan_x_begin) * (double) ix
        / (double) (num_shiftscan_x - 1);
  }
  for(iy=0; iy<num_shiftscan_y; iy++){
    G_shift[iy] = shiftscan_y_begin +
      (shiftscan_y_end - shiftscan_y_begin) * (double) iy
        / (double) (num_shiftscan_y - 1);
  }
  /* ==> Check ei and sr */
  ng = ei->num_rg;
  it = 0;  num_avrpix = 0;
  nx_pixels = sr->hpix_max - sr->hpix_min + 1;
  ny_pixels = sr->gpix_max - sr->gpix_min + 1;
  for(ig=0; ig<ng; ig++){
    progress_indicator(ig, ng);
    if(
      sr->cal_g[ig].h == ei->obs_rg[ig].h &&
      sr->cal_g[ig].k == ei->obs_rg[ig].k &&
      sr->cal_g[ig].l == ei->obs_rg[ig].l
      ){
      for(ix=0; ix<num_shiftscan_x; ix++){
        for(iy=0; iy<num_shiftscan_y; iy++){
          interpolate2d_of_1d_array(
            sr->intensity[ig], sr->num_k_points, sr->hpix, sr->gpix,
            nx_pixels, ny_pixels, sr->H, sr->G,
            ei->cal_intensity[ig][it], ei->num_data_points[ig],
            ei->obs_H[ig][it], ei->obs_G[ig][it],
            H_shift[ix], G_shift[iy],
            num_avrpix, avrpix_wt /* << dummy */
            );
          scale[ix][iy] = scale_factor_disk(
            ei->num_data_points[ig], ei->cal_intensity[ig][it],
            ei->obs_sig[ig][it], ei->obs_intensity[ig][it]
            );
          gof_shiftscan[ix][iy] =
            sqrt(
              chi_square_disk(
                ei->num_data_points[ig], ei->cal_intensity[ig][it],
                ei->obs_sig[ig][it], ei->obs_intensity[ig][it],
                scale[ix][iy]
                ) / (double) ei->num_data_points[ig]
              );
        }
      }
      /*
    norm_cross_correlation(
      org_image_width, org_image_height, org_image,
      tmpl_image_width, tmpl_image_height, tmpl_image,
      cross_corr_image
      );
       */
      find_gof_min(
        gof_shiftscan, num_shiftscan_x, num_shiftscan_y,
        &min_ix, &min_iy, &(success_flag[ig])
        );
      Scale_opt[ig] = scale[min_ix][min_iy];
      gof_reflex[ig] = gof_shiftscan[min_ix][min_iy];
      Hshift_opt[ig] = H_shift[min_ix];
      Gshift_opt[ig] = G_shift[min_iy];
    }
    else{
      printf("$$ Disagreement was found.\n");
      printf(
        "$$  sr->cal_g[%d] (=%d %d %d)\n", ig, 
        sr->cal_g[ig].h, sr->cal_g[ig].k, sr->cal_g[ig].l
        );
      printf(
        "$$  ei->obs_rg[%d] (=%d %d %d)\n", ig, 
        ei->obs_rg[ig].h, ei->obs_rg[ig].k, ei->obs_rg[ig].l
        );
      printf("$$ Abort.\n");
      exit(EXIT_FAILURE);
      }
  }
  printf("100%%\n");
  free_vector(H_shift);
  free_vector(G_shift);
  free_matrix(scale);
  free_matrix(gof_shiftscan);
}


#if 0
void  shiftscan_sim_int(
  char *simfile, char *intfile,
  int num_shiftscan_H, double Hshift_min, double Hshift_max,
  int num_shiftscan_G, double Gshift_min, double Gshift_max,
  double thickness,
  int num_reflex, HKL *g_reflex,
  double *Hshift_opt, double *Gshift_opt, 
  double *gof_reflex, double *scale_opt, int *success_flag,
  double *total_gof, double *total_scale_factor
  )
{
  int  ig;
  double  gof, scalef;
  FIT_EXP_INTSET  ei;
  SIM_RES  sr;
  
  read_int_file(&ei, intfile);
  read_sim_data(&sr, simfile);
  if(thickness != sr.thickness){
    fprintf(stdout, "$$ Disagreement in thickness. Abort.\n");
    exit(EXIT_FAILURE);
  }
  if(num_reflex != sr.num_cal_g){
    fprintf(stdout, "$$ Disagreement in num_reflex. Abort.\n");
    exit(EXIT_FAILURE);
  }
  
  set_calint_to_ei_shiftscan(
    &ei, &sr,
    num_shiftscan_H, Hshift_min, Hshift_max,
    num_shiftscan_G, Gshift_min, Gshift_max,
    num_reflex, Hshift_opt, Gshift_opt, scale_opt, success_flag
  );
  
  set_calint_to_ei_shift(
    &ei, &sr, num_reflex, Hshift_opt, Gshift_opt
    );
  *total_scale_factor = eval_scale_factor( &ei );
  *total_gof = eval_gof( &ei, *total_scale_factor );
  for(ig=0; ig<num_reflex; ig++){
    g_reflex[ig] = ei.obs_rg[ig];
  }
  
  free_ei(&ei);
  free_sr(&sr);
}
#endif


void shiftscan_res_output(
  char *shift_outfile, int num_reflex, double thickness, 
  double *Hshift_opt, double *Gshift_opt, HKL *g_reflex,
  double *gof_reflex, int *success_flag,
  double total_gof_shift, double total_scale_factor_shift
  )
{
  int  ig;
  double  art_weight;
  FILE  *fo;
  
  fo = fopen_chk_wmode(shift_outfile);
  art_weight = 1.0;
  
  fprintf(fo, "%% %s\n", shift_outfile);
  fprintf(fo, "%% total_gof_shift = %lf\n", total_gof_shift);
  fprintf(fo, "%% total_scale_factor_shift = %lf\n", total_scale_factor_shift);
  fprintf(fo, "REFLEX_SHIFT_PAR{\n");
  for(ig=0; ig<num_reflex; ig++){
    fprintf(fo, "%%\n");
    fprintf(fo, "%% hkl no. %d (gof_disk=%.1lf)\n", ig, gof_reflex[ig]);
    fprintf(
      fo, "%d  %d  %d  %.1lf\n",
      g_reflex[ig].h, g_reflex[ig].k, g_reflex[ig].l, art_weight
      );
    fprintf(fo, "%lf  ADJ  H%d  %%Hshift\n", Hshift_opt[ig], ig);
    fprintf(fo, "%lf  ADJ  G%d  %%Gshift\n", Gshift_opt[ig], ig);
  }
  fprintf(fo, "}REFLEX_SHIFT_PAR\n");
  fclose(fo);
}


/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
#if 0
void ed_sim_cbed_pattern(
  CRYSTAL_STRCT_DATA *csd, FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei,
  complex **vg, int num_thickness, double *thickness,
  int nk, int nkcal, R_HKL *K_xy, HGPIX *K_pix,
  double ***intensity
  )
{
  int  ig, ng, nr, ik, it, nt;
  int  *tmp_obs_rg;
  int  num_perturb, its;
  int  exec_time_flag;
  double  abs_incidence, abs_surf_norm, cosfai_inc_surf;
  double  wave_length, ev_dash, abs_K, g;
  double  scale, tmp;
  double  **tmp_intensity, ***tmp_perturb_intensity;
  double  cal_int, obs_int, obs_sig, sig2i, dy, chisq;
  double  su, sd;
  complex  ***perturbed_vgs;
  CELL_PAR  bv;
  WORK_CAL_INT  wci;

  num_perturb = 0;  /* No jacobian calc. */

  bv = bravai(csd->lp);
  pre_calc_intensity(
    ed->accel_kvol, ed->incidence, ed->surf_norm, bv,
    &abs_incidence, &abs_surf_norm, &cosfai_inc_surf, &wave_length, &ev_dash
    );

  ng = ed->num_rg;   /* number of obs. reflections */
  nt = ed->num_thickness;
  if( nt > 1 ){
    mb_fprintf(stdout,
      "$$ (TSCAN_MODE) num. of thickness of exp. data must be 1. Abort.\n"
      );
    exit(EXIT_FAILURE);
  }
  nr = ed->num_exact + ed->num_bethe;

  tmp_obs_rg = new_vector(int, ng);   chk_vector(tmp_obs_rg);
  tmp_intensity = new_matrix(double, ng, num_thickness);
  chk_matrix(tmp_intensity);
  alloc_work_cal_int(nr, ng, num_perturb, &wci);

  abs_K = calc_abs_K( csd, ed->accel_kvol, bv );

  /* Caution: nk must be equal not to ei[ig]->num_data_points
     but sq(num_points_h) */

  for(ig=0; ig<ng; ig++){
    tmp_obs_rg[ig] =
      find_n_g(ed->num_exact + ed->num_bethe, ei->obs_rg[ig], ed->reflex);
    tmp = sq_abs(hkl2rhkl(ei->obs_rg[ig]), bv);	ed->g[ig] = sqrt(tmp);
    if(tmp_obs_rg[ig] == -1){
      mb_fprintf(stdout,
        "$$ ed_sim_cbed_pattern: %d %d %d not found in reflex[]. Abort.",
        ei->obs_rg[ig].h, ei->obs_rg[ig].k, ei->obs_rg[ig].l
        );
      exit(EXIT_FAILURE);
    }
  }

  mb_fprintf(stdout, "$$ nk=%ld\n", nk);
  mb_fprintf(stdout, "Intensity calcultion....\n");
  exec_time_flag = 0;

  for(ik=0; ik<nk; ik++){
    progress_indicator(ik, nk);
    if(K_pix[ik].flag == 1){
      if(exec_time_flag==0) {
        exec_time_forecast(-1);
      }
      calc_intensity_with_perturb_ug(
        K_xy[ik], abs_K,
        ed->beam_search, ed->num_exact, ed->num_bethe, ed->reflex,
        vg, num_perturb, perturbed_vgs, ng, tmp_obs_rg,
        num_thickness, thickness,
        ed->accel_kvol, ed->incidence, ed->surf_norm, bv,
        abs_incidence, abs_surf_norm, cosfai_inc_surf,
        tmp_intensity, tmp_perturb_intensity, &wci
        );
      for(ig=0; ig<ng; ig++){
        for(its=0; its<num_thickness; its++){
          intensity[ig][its][ik] = tmp_intensity[ig][its];
        }
      }
      if(exec_time_flag==0){
        exec_time_forecast(nkcal);
        exec_time_flag = 1;
      }
    }
    else{
      for(ig=0; ig<ng; ig++){
        for(its=0; its<num_thickness; its++){
          intensity[ig][its][ik] = 0.0;
        }
      }
    }
  }
  exec_time_forecast(0);
  free_work_cal_int(&wci);
  free_vector(tmp_obs_rg);
  free_vector(tmp_intensity);
}


/* TSCAN_MODE ------------------------------------------------------------- */
/* ------------------------------------------------------------------------ */
void eval_tscan_chisq(
  FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei, double ***intensity,
  int num_tscan, double *tscan, double *chisq_tscan, double *scalef,
  int num_points_h, int nk, R_HKL *K_xy, HGPIX *K_pix
  )
{
  /*
   ***** t-scan *****
   */
  double  scale, **itp_intensity, g, tmp;
  double  su, sd;
  double  cal_int, obs_int, obs_sig, sig2i, dy, chisq;
  int     ig, ng, it, its, ik; 

  mb_fprintf(stdout, "Thickness scan....\n");
  ng = ed->num_rg;   /* number of obs. reflections */
  /* itp_intensity: interpolated intensities */
  itp_intensity = (double **) new_matrix_element(ng);
  chk_matrix(itp_intensity);
  for(ig=0; ig<ng; ig++){
    itp_intensity[ig] = new_vector_element(double, ei->num_data_points[ig]);
    chk_matrix(itp_intensity);
  }

  it = 0; /* it: ith-thickness of experimental intensities */
  for(its=0; its<num_tscan; its++){	/* its: t-scan */
    /* Transformation from cal to obs by interpolation */
    for(ig=0; ig<ng; ig++){
      g = ed->g[ig];
      interpolation_2d(
        itp_intensity[ig], intensity[ig][its], K_xy, K_pix,
        ed->H_tilt + ed->H_shift[ig][it], ed->G_tilt + ed->G_shift[ig][it],
        ei->obs_H[ig][it], ei->obs_G[ig][it],
        ei->num_data_points[ig], num_points_h, nk,
        ed->num_avrpix[it], ed->avrpix_wt[it]
        );
    }

    /* Evaluation of scale factors */
    su = 0.0; sd = 0.0;
    for(ig=0; ig<ng; ig++){
      g = ed->g[ig];
      for(ik=0; ik<ei->num_data_points[ig]; ik++){
        cal_int = itp_intensity[ig][ik] * epsilon.intensity_scaling;
        obs_sig = ei->obs_sig[ig][it][ik];
        obs_int = ei->obs_intensity[ig][it][ik];
        /*	sig2i = cal_weight_tscan(obs_sig, obs_int, cal_int, g);*/
        sig2i = cal_weight(obs_sig, obs_int, cal_int, g, 1.0);
        su += obs_int * cal_int * sig2i;
        sd += sq(cal_int) * sig2i;
      }
    }

    if(sd > epsilon.eps){
      scale = su / sd;
    }
    else{
      mb_fprintf(
        stdout, "$$ (TSCAN_MODE) Too small denominator (%lf). Abort.\n", sd
        );
      exit(EXIT_FAILURE);
    }
    scalef[its] = scale;

    /* Calc. of Chi-squares */
    chisq = 0.0;
    for(ig=0; ig<ng; ig++){
      g = ed->g[ig];
      for(ik=0; ik<ei->num_data_points[ig]; ik++){
        cal_int = itp_intensity[ig][ik] * epsilon.intensity_scaling * scale;
        obs_sig = ei->obs_sig[ig][it][ik];
        obs_int = ei->obs_intensity[ig][it][ik];
        /*	sig2i = cal_weight_tscan(obs_sig, obs_int, cal_int, g);*/
        sig2i = cal_weight(obs_sig, obs_int, cal_int, g, 1.0);
        dy = obs_int - cal_int;
        chisq += sq(dy) * sig2i;
        if(dbg.obs_cal_dump_flag == 1){	/* debug */
          mb_fprintf(
            stdout, "$$ obs, cal = %lf, %lf\n", obs_int, cal_int
            );
        }
      }
    }
    chisq_tscan[its] = chisq;
  }
  free_matrix(itp_intensity);
}


void dynamical_calc_tscan(
  CRYSTAL_STRCT_DATA *csd,
  int ith_expdata, FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei,
  int num_tscan, double *tscan, double *chisq_tscan, double *scalef
  )
{
  int     nr, nksize, ng, num_points_h, nk, nkcal;
  double  scale, ***intensity, **itp_intensity, tmp;
  double  G_bgn, G_end, H_bgn, H_end;
  R_HKL   *K_xy;
  HGPIX   *K_pix;
  complex  **vg;
  CELL_PAR bv;

  /* ----- Beam search ----- */
  /*
  if(
    ed->beam_search.flag_bs == 1 || ed->beam_search.flag_bs == 11 ||
    ed->beam_search.flag_bs == 2 || ed->beam_search.flag_bs == 12 ||
    ed->beam_search.flag_bs == 3 || ed->beam_search.flag_bs == 13
    ){*/
  if( ed->beam_search.flag_bs != 0 && ed->beam_search.flag_bs != 10 ){
    auto_ed_beams(ed, csd);
  }
  /* Debug */
  bv = bravai(csd->lp);
  reflex_dump_csd(
    ith_expdata, ed->num_exact, ed->num_bethe, ed->reflex, csd,
    ed->beam_search, ed->incidence, ed->zolzbase1, ed->accel_kvol, bv
    );

  /* ----- Potential matrix ----- */
  nr = ed->num_exact + ed->num_bethe;
  vg = new_matrix(complex, nr, nr);  chk_matrix(vg);
  mb_fprintf(stdout, "Potential...\n");  mb_fflush(stdout);
  potential_matrix_csd(
    ed->num_exact, ed->num_bethe, ed->reflex, csd, ed->accel_kvol, bv, vg
    );
  debug_vg_dump( vg, ed->num_exact + ed->num_bethe );

  nksize_evaluation(ed, &G_bgn, &G_end, &H_bgn, &H_end, &num_points_h);
  nksize = sq(num_points_h);
  ng = ed->num_rg;   /* number of obs. reflections */
  intensity  = new_tensor(double, ng, num_tscan, nksize);
  chk_tensor(intensity);
  K_xy = new_vector(R_HKL, nksize);   chk_vector(K_xy);
  K_pix = new_vector(HGPIX, nksize);  chk_vector(K_pix);

  calc_K_xy_cbed(
    &nk, &nkcal, K_xy, K_pix, ed->H_tilt, ed->G_tilt,
    ed->incidence, ed->zolzbase1, bv, ed->calc_type,
    G_bgn, G_end, H_bgn, H_end, num_points_h
    );

  ed_sim_cbed_pattern(
    csd, ed, ei, vg, num_tscan, tscan, nk, nkcal, K_xy, K_pix, intensity
    );

  /* TSCAN */
  eval_tscan_chisq(
    ed, ei, intensity, num_tscan, tscan, chisq_tscan, scalef,
    num_points_h, nk, K_xy, K_pix
  );

  free_matrix(vg);
  free_vector(K_xy);
  free_vector(K_pix);
  free_tensor(intensity);
  if( ed->beam_search.flag_bs != 0 && ed->beam_search.flag_bs != 10 ){
    free_vector(ed->reflex);
  }
}


void input_tscan_exp_dataset(
  FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei,
  int *num_tscan, double *tscan_bgn, double *tscan_end, FILE *fi
  )
{
  /*
   * Input fit_exp_dataset from fi
   *  for a element of fit_exp_dataset
   *  *ed : pointer (not an array)
   */
  int it, nt, ig, ng, ik, nd, tmp_nt, tmp_ng;
  HKL tmp_g;
  char buf[BUFLEN];
  FILE *fint;

  /* INCIDENT_BEAM block */
  chk_data("INCIDENT_BEAM{", fi);
  ed->incidence = fget_uvw(fi);
  ed->surf_norm = fget_uvw(fi);
  ed->zolzbase1 = fget_r_hkl(fi);
  fget_fitpar(&(ed->H_tilt_f), &(ed->H_tilt), fi);
  fget_fitpar(&(ed->G_tilt_f), &(ed->G_tilt), fi);
  fget_fitpar(&(ed->accel_kvol_f), &(ed->accel_kvol), fi);
  chk_data("}INCIDENT_BEAM", fi);

  /* BEAM_SEARCH block */
  chk_data("BEAM_SEARCH{", fi);
  fget_ed_beams(ed, fi);
  chk_data("}BEAM_SEARCH", fi);

  /* CALC_KXY_RANGE block */
  chk_data("CALC_KXY_RANGE{", fi);
  ed->calc_range_flag = fget_i(fi);
  ed->calc_type = fget_i(fi);
  if(
    ed->calc_type == 0 || ed->calc_type == 1 ||
    ed->calc_type == 2 ||  ed->calc_type == 3
    ){
    ed->calc_numh = fget_i(fi);
    fget_2d( &(ed->calc_hbgn), &(ed->calc_hend), fi );
    fget_2d( &(ed->calc_gbgn), &(ed->calc_gend), fi );
  }
  else{
    mb_fprintf(stdout, "$$ Invalid calc_type=%d. Abort.\n", ed->calc_type);
    exit(EXIT_FAILURE);
  }
  chk_data("}CALC_KXY_RANGE", fi);

  chk_data("TSCAN_WIDTH{", fi);
  *num_tscan = fget_i(fi);
  fget_2d( tscan_bgn, tscan_end, fi );
  chk_data("}TSCAN_WIDTH", fi);

  /* INTENSITY_DATA block */
  chk_data("INTENSITY_DATA{", fi);
  fget_1s( ed->intensity_file, fi );
  if( (fint = fopen(ed->intensity_file, "r")) == NULL ){
    mb_fprintf(stdout, "$$ Cannot open %s. Abort.\n", ed->intensity_file);
    exit(EXIT_FAILURE);
  }

  nt = ed->num_thickness = ei->num_thickness = fget_i(fi);
  tmp_nt = fget_i(fint);
  mb_fprintf(stdout, "$$ nt, tmp_nt = %d, %d\n", nt, tmp_nt);
  if( nt != tmp_nt ){
    mb_fprintf(stdout, 
      "$$ num_thickness = %d is not equal to int-file's = %d. Abort.\n",
      nt, tmp_nt
      );
    exit(EXIT_FAILURE);
  }

  ed->scale_factor = new_vector(double, nt);
  chk_vector(ed->scale_factor);
  ed->scale_factor_f = new_vector(FIT_CTRL, nt);
  chk_vector(ed->scale_factor_f);
  ed->thickness = new_vector(double, nt);
  chk_vector(ed->thickness);
  ed->thickness_f = new_vector(FIT_CTRL, nt);
  chk_vector(ed->thickness_f);
  ed->thickwidth = new_vector(double, nt);
  chk_vector(ed->thickwidth);
  ed->thickwidth_f = new_vector(FIT_CTRL, nt);
  chk_vector(ed->thickwidth_f);
  ed->num_avrpix = new_vector(int, nt);
  chk_vector(ed->num_avrpix);/*dummy*/
  ed->gaussian_par = new_vector(double, nt);
  chk_vector(ed->gaussian_par);/*dummy*/
  ed->avrpix_wt = new_tensor_element(nt);
  chk_tensor(ed->avrpix_wt);/*dummy*/

  /* ng: number of reflections of experimental data */
  ng = ed->num_rg = ei->num_rg = fget_i(fi);
  tmp_ng = fget_i(fint);
  if( ng != tmp_ng ){
    mb_fprintf(stdout,
      "$$ Num. of reflex. = %d is not equal to the int file's = %d. Abort.\n",
      ng, tmp_ng
      );
    exit(EXIT_FAILURE);
  }
  chk_data("}INTENSITY_DATA", fi);

  ei->obs_rg = new_vector(HKL, ng);         chk_vector(ei->obs_rg);
  ed->g = new_vector(double, ng);           chk_vector(ed->g);
  ed->art_weight = new_vector(double, ng);  chk_vector(ed->art_weight);

  ed->H_shift = new_matrix(double, ng, nt);     chk_matrix(ed->H_shift);
  ed->G_shift = new_matrix(double, ng, nt);     chk_matrix(ed->G_shift);
  ed->H_shift_f = new_matrix(FIT_CTRL, ng, nt); chk_matrix(ed->H_shift_f);
  ed->G_shift_f = new_matrix(FIT_CTRL, ng, nt); chk_matrix(ed->G_shift_f);

  ei->num_data_points = new_vector(int, ng);
  chk_vector(ei->num_data_points);

  ei->obs_H = (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_H);
  ei->obs_G = (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_G);
  ei->obs_intensity	= (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_intensity);
  ei->cal_intensity	= (double ***) new_tensor_element(ng);
  chk_tensor(ei->cal_intensity);
  ei->obs_sig		= (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_sig);

  for(ig=0; ig<ng; ig++){
    ei->obs_H[ig]  = (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_H[ig]);
    ei->obs_G[ig]  = (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_G[ig]);
    ei->obs_intensity[ig]  = (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_intensity[ig]);
    ei->cal_intensity[ig]  = (double **) new_matrix_element(nt);
    chk_matrix(ei->cal_intensity[ig]);
    ei->obs_sig[ig]  = (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_sig[ig]);
  }

  for(it=0; it<nt; it++){
    /* THICKNESS_BLOCK part */
    chk_data("THICKNESS_BLOCK{", fi);
    ed->num_avrpix[it] = 0; /* dummy */

    /* REFLEX_SHIFT_PAR block */
    chk_data("REFLEX_SHIFT_PAR{", fi);
    for(ig=0; ig<ng; ig++){
      ei->obs_rg[ig] = fget_hkl(fint);
      tmp_g = fget_hkl(fi);
      /*
      mb_fprintf(stdout, "$$ tmp_g: %d %d %d\n", tmp_g.h, tmp_g.k, tmp_g.l);
       */
      if( 
        ei->obs_rg[ig].h != tmp_g.h ||
        ei->obs_rg[ig].k != tmp_g.k ||
        ei->obs_rg[ig].l != tmp_g.l 
        ) {
        mb_fprintf(stdout, "$$ hkl is inconsistent: \n");
        mb_fprintf(stdout, 
          "$$ par:%d %d %d : int:%d %d %d.    Abort.\n",
          tmp_g.h, tmp_g.k, tmp_g.l,
          ei->obs_rg[ig].h, ei->obs_rg[ig].k, ei->obs_rg[ig].l
          );
        exit(EXIT_FAILURE);
      }

      fget_fitpar(&(ed->H_shift_f[ig][it]), &(ed->H_shift[ig][it]), fi);
      fget_fitpar(&(ed->G_shift_f[ig][it]), &(ed->G_shift[ig][it]), fi);
      nd = ei->num_data_points[ig] = fget_i(fint);

      ei->obs_H[ig][it]  = new_vector_element(double, nd);
      chk_vector(ei->obs_H[ig][it]);
      ei->obs_G[ig][it]  = new_vector_element(double, nd);
      chk_vector(ei->obs_G[ig][it]);
      ei->obs_intensity[ig][it] = new_vector_element(double, nd);
      chk_vector(ei->obs_intensity[ig][it]);
      ei->cal_intensity[ig][it] = new_vector_element(double, nd);
      chk_vector(ei->cal_intensity[ig][it]);
      ei->obs_sig[ig][it] = new_vector_element(double, nd);
      chk_vector(ei->obs_sig[ig][it]);

      for(ik=0; ik<nd; ik++){
        fget_4d(
          &(ei->obs_H[ig][it][ik]), &(ei->obs_G[ig][it][ik]),
          &(ei->obs_intensity[ig][it][ik]), &(ei->obs_sig[ig][it][ik]),
          fint
          );
        if(ei->obs_sig[ig][it][ik] <= 0.0){	/* <<<<< */
          mb_fprintf(stdout, "$$ warning !!: sig <= 0.0\n");
        }
      }
      chk_data("RF_END",fint);
    }
    chk_data("}REFLEX_SHIFT_PAR", fi);
    chk_data("}THICKNESS_BLOCK", fi);
    chk_data("THICK_END",fint);
  }
  chk_data("INTENSITY_DATA_END",fint);
  fclose(fint);
}


void mb_thickness_scan( FILE *fi, char *infile )
{
  /*
    Thickness scan:
   */
  CRYSTAL_STRCT_DATA csd;
  FIT_EXP_DATASET *fed;
  FIT_EXP_INTSET  *fei;
  double tscan_bgn, tscan_end, *tscan, *chisq_tscan, *scalef;
  int    num_exp_dataset, ie, num_tscan, its, ig, ng, ik, nk;
  long   ct, total_num_data;
  char   log_outfile[BUFLEN];
  FILE   *fpo;

  input_csd_cstr(&csd, fi, infile);    /* crystal structure data */

  /* ----- Read experimental data ----- */
  chk_data("TSCAN_CALCULATION_CONDITION{", fi);
  num_exp_dataset = 1;
  chk_data("DATA_SET{", fi);
  fed = new_vector(FIT_EXP_DATASET, num_exp_dataset);  chk_vector(fed);
  fei = new_vector(FIT_EXP_INTSET, num_exp_dataset);   chk_vector(fei);
  ie = 0;
  input_tscan_exp_dataset(
    &(fed[ie]), &(fei[ie]), &num_tscan, &tscan_bgn, &tscan_end, fi
    );
  chk_data("}DATA_SET", fi);
  chk_data("}TSCAN_CALCULATION_CONDITION", fi);
  fclose(fi);
  constraint_fed(num_exp_dataset, fed);

  /* ----- Set t-scan ----- */
  if(num_tscan <= 1){
    mb_fprintf(stdout, "$$ num_tscan must be greater than 1. Abort.\n");
    exit(EXIT_FAILURE);
  }
  tscan  = new_vector(double, num_tscan);      chk_vector(tscan);
  chisq_tscan = new_vector(double, num_tscan); chk_vector(chisq_tscan);
  scalef = new_vector(double, num_tscan);      chk_vector(scalef);
  for(its=0; its<num_tscan; its++){
    tscan[its] = tscan_bgn +
      (double)its * (tscan_end - tscan_bgn) / (double)(num_tscan - 1);
  }

  dynamical_calc_tscan(
    &csd, ie, &(fed[ie]), &(fei[ie]),
    num_tscan, tscan, chisq_tscan, scalef
    );

  /* Number of total data points */
  ct = 0;
  ng = fed[ie].num_rg;
  for(ig=0; ig<ng; ig++){
    nk = fei[ie].num_data_points[ig];
    for(ik=0; ik<nk; ik++){
      ct++;
    }
  }
  total_num_data = ct;
  mb_fprintf(stdout, "$$ total_num_data = %ld\n", total_num_data);
  if( total_num_data <= 0 ){
    mb_fprintf(stdout, "$$ No data points. Abort.\n");
    exit(EXIT_FAILURE);
  }

  /* ----- Result output ----- */
  sprintf(log_outfile, "%s_trs.txt", infile);
  mb_fprintf(stdout, "Result of thickness scan ==> %s\n", log_outfile);
  if( (fpo = fopen(log_outfile,"w")) == NULL ){
    mb_fprintf(stdout, "$$ Cannot open %s\n", log_outfile);
    exit(EXIT_FAILURE);
  }
  mb_fprintf(fpo, "%% Thickness scan\n");
  mb_fprintf(fpo, "%% %s.cr and %s.cal\n", infile, infile);
  mb_fprintf(fpo, "%%t  GOF  scalefactor\n");
  for(its=0; its<num_tscan; its++){
    mb_fprintf(
      fpo, "%lf  %lf  %lf\n",
      tscan[its], sqrt(chisq_tscan[its] / (double) total_num_data),
      scalef[its]
      );
  }
  fclose(fpo);

  free_vector(chisq_tscan);
  free_vector(tscan);
  free_vector(scalef);
  free_fed_vector(num_exp_dataset, fed);
  free_fei_vector(num_exp_dataset, fei);
  free_csd(&csd);
}


/* SHIFTSCAN_MODE --------------------------------------------------------- */
/* ------------------------------------------------------------------------ */


void norm_cross_correlation(
  int org_image_width, int org_image_height, double **org_image,
  int tmpl_image_width, int tmpl_image_height, double **tmpl_image,
  double **cross_corr_image
  )
{
  /*
    Normalized cross correlation coefficient:
     Image_width and _heigth of cross_corr_image[][] are the same as
     those of org_image[][].
 */

  int i, j, m, n;
  double tmpu, tmpd1, tmpd2, val_org, val_tmpl;

  /* Check image sizes. */
  if(org_image_width < tmpl_image_width){
    mb_fprintf(stdout, "$$ org_image_width = %d should be greater than\n");
    mb_fprintf(stdout, 
      "$$  tmpl_image_width=%d. Abort.\n", org_image_width, tmpl_image_width
      );
    exit(EXIT_FAILURE);
  }
  if(org_image_height < tmpl_image_height){
    mb_fprintf(stdout, "$$ org_image_height = %d should be greater than\n");
    mb_fprintf(stdout, 
      "$$ tmpl_image_heigth=%d. Abort.\n", org_image_height, tmpl_image_height
      );
    exit(EXIT_FAILURE);
  }

  /* Initialize */
  for(m=0; m<org_image_width; m++){
    for(n=0; n<org_image_height; n++){
      cross_corr_image[m][n] = 0.0;
    }
  }

  tmpu = 0.0; tmpd1 = 0.0; tmpd2 = 0.0;
  for(m=0; m<org_image_width-tmpl_image_width; m++){
    for(n=0; n<org_image_height-tmpl_image_height; n++){
      /* Cross correlation */
      for(i=m; i<tmpl_image_width+m; i++){
        for(j=n; j<tmpl_image_height+n; j++){
          val_org = org_image[i][j];
          val_tmpl = tmpl_image[i-m][j-n];
          tmpu  += val_org * val_tmpl;
          tmpd1 += sq(val_org);
          tmpd2 += sq(val_tmpl);
        }
      }
      cross_corr_image[m][n] = tmpu / ( sqrt(tmpd1) * sqrt(tmpd2) );
    }
  }

}


double scale_factor_disk(
  int num_points, double *cal_int, double *obs_sig, double *obs_int,
  double g
  )
{
  /* Scale factor for one disk */
  int  ik;
  double  su, sd, c_int, o_sig, o_int, sig2i, scale;

  su = 0.0; sd = 0.0;
  for(ik=0; ik<num_points; ik++){
    c_int = cal_int[ik] * epsilon.intensity_scaling;
    o_sig = obs_sig[ik];
    o_int = obs_int[ik];
    /*	sig2i = cal_weight_tscan(obs_sig, obs_int, cal_int, g);*/
    sig2i = cal_weight(o_sig, o_int, c_int, g, 1.0);
    su += o_int * c_int * sig2i;
    sd += sq(c_int) * sig2i;
  }

  if(sd > epsilon.eps){
    scale = su / sd;
  }
  else{
    mb_fprintf(stdout, 
      "$$ (TSCAN_MODE: scale_factor_disk) Too small denominator (%lf).\n", sd
      );
    mb_fprintf(stdout, "$$ scale is set to be 1.0.\n");
    scale = 1.0;
    /* exit(EXIT_FAILURE); */
  }
  return scale;
}


double chi_square_disk(
  int num_points, double *cal_int, double *obs_sig, double *obs_int,
  double g, double scale
  )
{
  /* Calc. of Chi-squares */
  int  ik;
  double  c_int, o_sig, o_int, sig2i, dy, chisq;

  chisq = 0.0;
  for(ik=0; ik<num_points; ik++){
    c_int = cal_int[ik] * epsilon.intensity_scaling * scale;
    o_sig = obs_sig[ik];
    o_int = obs_int[ik];
    /* sig2i = cal_weight_tscan(obs_sig, obs_int, cal_int, g); */
    sig2i = cal_weight(o_sig, o_int, c_int, g, 1.0);
    dy = o_int - c_int;
    chisq += sq(dy) * sig2i;
    if(dbg.obs_cal_dump_flag == 1){/* debug */
      mb_fprintf( stdout, "$$ obs, cal = %lf, %lf\n", o_int, c_int );
    }
  }
  return chisq;
}


void find_gof_min(
  double **gof_shiftscan, int num_shiftscan, int *min_ibigh, int *min_ibigg
  )
{
  int ibigh, ibigg;
  double min_gof;

  min_gof = 1e10;
  *min_ibigh = 0; *min_ibigg = 0;  /* <<<<< dummy for flat chisq */
  for(ibigg=0; ibigg<num_shiftscan; ibigg++){
    for(ibigh=0; ibigh<num_shiftscan; ibigh++){
      if(min_gof > gof_shiftscan[ibigh][ibigg]){
        min_gof = gof_shiftscan[ibigh][ibigg];
        *min_ibigh = ibigh;
        *min_ibigg = ibigg;
      }
    }
  }
}


void eval_shiftscan_gof(
  FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei, double ***intensity,
  int num_shiftscan, double shiftscan_bgn, double shiftscan_end,
  int num_points_h, int nk, R_HKL *K_xy, HGPIX *K_pix,
  char *infile
  )
{
  /*
    shift-scan 
   */
  double  **itpl_intensity, g, tmp, gof_min;
  double  **gof_shiftscan, *Hshift_add, *Gshift_add, **scale;
  int     ibigh, ibigg, min_ibigh, min_ibigg;
  int     ig, ng, it, ik;
  char    outfile[BUFLEN];
  FILE    *fp;

  mb_fprintf(stdout, "Shift scan....\n");
  sprintf(outfile, "%s_sf.txt", infile);
  if( (fp = fopen(outfile, "w")) == NULL ){
    mb_fprintf(stdout, "$$ Cannot open %s. Abort.\n", outfile);
    exit(EXIT_FAILURE);
  }

  ng = ed->num_rg;   /* number of obs. reflections */

  /* itpl_intensity: interpolated intensities */
  itpl_intensity = (double **) new_matrix_element(ng);
  chk_matrix(itpl_intensity);
  for(ig=0; ig<ng; ig++){
    itpl_intensity[ig] = new_vector_element(double, ei->num_data_points[ig]);
    chk_matrix(itpl_intensity);
  }

  gof_shiftscan = new_matrix(double, num_shiftscan, num_shiftscan);
  chk_matrix(gof_shiftscan);
  scale = new_matrix(double, num_shiftscan, num_shiftscan);
  chk_matrix(scale);
  Hshift_add = new_vector(double, num_shiftscan);
  chk_vector(Hshift_add);
  Gshift_add = new_vector(double, num_shiftscan);
  chk_vector(Gshift_add);

  it = 0; /* it: ith-thickness of experimental intensities */
  for(ig=0; ig<ng; ig++){
    g = ed->g[ig];
    mb_fprintf(stdout, "$$ig=%d\n", ig);
    for(ibigg=0; ibigg<num_shiftscan; ibigg++){
      Gshift_add[ibigg] = shiftscan_bgn
        + (shiftscan_end - shiftscan_bgn)
          * (double) ibigg / (double) (num_shiftscan - 1);
      for(ibigh=0; ibigh<num_shiftscan; ibigh++){
        Hshift_add[ibigh] = shiftscan_bgn
          + (shiftscan_end - shiftscan_bgn)
            * (double) ibigh / (double) (num_shiftscan - 1);
        interpolation_2d(
          itpl_intensity[ig], intensity[ig][it], K_xy, K_pix,
          ed->H_tilt + ed->H_shift[ig][it] + Hshift_add[ibigh],
          ed->G_tilt + ed->G_shift[ig][it] + Gshift_add[ibigg],
          ei->obs_H[ig][it], ei->obs_G[ig][it],
          ei->num_data_points[ig], num_points_h, nk,
          ed->num_avrpix[it], ed->avrpix_wt[it]
          );
        scale[ibigh][ibigg] = scale_factor_disk(
          ei->num_data_points[ig], itpl_intensity[ig],
          ei->obs_sig[ig][it], ei->obs_intensity[ig][it], g
          );
        gof_shiftscan[ibigh][ibigg] =
          sqrt(
            chi_square_disk(
              ei->num_data_points[ig], itpl_intensity[ig],
              ei->obs_sig[ig][it], ei->obs_intensity[ig][it],
              g, scale[ibigh][ibigg]
              ) / (double) ei->num_data_points[ig]
            );
      }
    }

    find_gof_min(
      gof_shiftscan, num_shiftscan, &min_ibigh, &min_ibigg
      );
    mb_fprintf(
      fp, "ig:%d, hkl=%d %d %d:\n",
      ig, ei->obs_rg[ig].h,  ei->obs_rg[ig].k,  ei->obs_rg[ig].l
      );
    mb_fprintf(
      fp, " GoF min.=%lf at (H,G)=(%lf, %lf), scale=%lf.\n\n",
      gof_shiftscan[min_ibigh][min_ibigg],
      ed->H_shift[ig][it] + Hshift_add[min_ibigh],
      ed->G_shift[ig][it] + Gshift_add[min_ibigg],
      /* ed->H_tilt + ed->H_shift[ig][it] + Hshitf_add[ibigh],
      ed->G_tilt + ed->G_shift[ig][it] + Gshift_add[ibigg] */
      scale[min_ibigh][min_ibigg]
      );
    /*
      print_chisq_shiftscan(
      num_shiftscan, chisq_shiftscan, Hshift_add, Gshift_add, ig, infile
      );
     */
    /*
    norm_cross_correlation(
      org_image_width, org_image_height, org_image,
      tmpl_image_width, tmpl_image_height, tmpl_image,
      cross_corr_image
      );
     */
  }

  fclose(fp);
  free_matrix(itpl_intensity);
  free_matrix(gof_shiftscan);
  free_vector(Hshift_add);  free_vector(Gshift_add);
}


void dynamical_calc_shiftscan(
  CRYSTAL_STRCT_DATA *csd,
  int ith_expdata, FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei,
  int num_shiftscan, double shiftscan_bgn, double shiftscan_end,
  char *infile
  )
{
  int     nr, nksize, ng, num_points_h, nk, nkcal, nt;
  int     i_bigh, i_bigg;
  double  scale, ***intensity, **itp_intensity, tmp, *thickness;
  double  G_bgn, G_end, H_bgn, H_end;
  R_HKL   *K_xy;
  HGPIX   *K_pix;
  complex  **vg;
  CELL_PAR bv;

  /* ----- Beam search ----- */
/*  if(
    ed->beam_search.flag_bs == 1 || ed->beam_search.flag_bs == 11 ||
    ed->beam_search.flag_bs == 2 || ed->beam_search.flag_bs == 12 ||
    ed->beam_search.flag_bs == 3 || ed->beam_search.flag_bs == 13
    ){ */
  if( ed->beam_search.flag_bs != 0 && ed->beam_search.flag_bs != 10 ){
    auto_ed_beams(ed, csd);
  }
  bv = bravai(csd->lp);
  reflex_dump_csd(
    ith_expdata, ed->num_exact, ed->num_bethe, ed->reflex, csd,
    ed->beam_search, ed->incidence, ed->zolzbase1, ed->accel_kvol, bv
    );

  /* ----- Potential matrix ----- */
  nr = ed->num_exact + ed->num_bethe;
  vg = new_matrix(complex, nr, nr);  chk_matrix(vg);
  mb_fprintf(stdout, "Potential...\n");  mb_fflush(stdout);
  potential_matrix_csd(
    ed->num_exact, ed->num_bethe, ed->reflex, csd, ed->accel_kvol, bv, vg
    );
  debug_vg_dump( vg, ed->num_exact + ed->num_bethe );

  nksize_evaluation(ed, &G_bgn, &G_end, &H_bgn, &H_end, &num_points_h);
  nksize = sq(num_points_h);
  ng = ed->num_rg;   /* number of obs. reflections */
  nt = 1;  /* thickness */
  intensity = new_tensor(double, ng, nt, nksize);
  chk_tensor(intensity);
  K_xy = new_vector(R_HKL, nksize);   chk_vector(K_xy);
  K_pix = new_vector(HGPIX, nksize);  chk_vector(K_pix);
  thickness = new_vector(double, nt); chk_vector(thickness);
  thickness[0] = ed->thickness[0];
  mb_fprintf(stdout, "$$ thickness[0]=%lf\n", thickness[0]);

  calc_K_xy_cbed(
    &nk, &nkcal, K_xy, K_pix, ed->H_tilt, ed->G_tilt,
    ed->incidence, ed->zolzbase1, bv, ed->calc_type,
    G_bgn, G_end, H_bgn, H_end, num_points_h
    );

  ed_sim_cbed_pattern(
    csd, ed, ei, vg, nt, thickness, nk, nkcal, K_xy, K_pix, intensity
    );

  /* SHIFTSCAN */
  eval_shiftscan_gof(
    ed, ei, intensity,
    num_shiftscan, shiftscan_bgn, shiftscan_end,
    num_points_h, nk, K_xy, K_pix,
    infile
  );

  free_matrix(vg);
  free_vector(K_xy);
  free_vector(K_pix);
  free_tensor(intensity);
  if( ed->beam_search.flag_bs != 0 && ed->beam_search.flag_bs != 10 ){
    free_vector(ed->reflex);
  }
}


void input_shiftscan_exp_dataset(
  FIT_EXP_DATASET *ed, FIT_EXP_INTSET *ei, 
  int *num_shiftscan, double *shiftscan_bgn, double *shiftscan_end,
  FILE *fi
  )
{
  /*
   * Input fit_exp_dataset from fi
   *  for a element of fit_exp_dataset
   *  *ed : pointer (not an array)
   */
  int it, nt, ig, ng, ik, nd, tmp_nt, tmp_ng;
  HKL tmp_g;
  char buf[BUFLEN];
  FILE *fint;

  /* INCIDENT_BEAM block */
  chk_data("INCIDENT_BEAM{", fi);
  ed->incidence = fget_uvw(fi);
  ed->surf_norm = fget_uvw(fi);
  ed->zolzbase1 = fget_r_hkl(fi);
  fget_fitpar(&(ed->H_tilt_f), &(ed->H_tilt), fi);
  fget_fitpar(&(ed->G_tilt_f), &(ed->G_tilt), fi);
  fget_fitpar(&(ed->accel_kvol_f), &(ed->accel_kvol), fi);
/*  fget_name_cstr_idx(
    ed->H_tilt_f.name, ed->H_tilt_f.cstr, &(ed->H_tilt_f.flg), fi
    );
  par_transval( ed->H_tilt_f.flg, ed->H_tilt_f.cstr, &(ed->H_tilt) );
  fget_name_cstr_idx(
    ed->G_tilt_f.name, ed->G_tilt_f.cstr, &(ed->G_tilt_f.flg), fi
    );
  par_transval( ed->G_tilt_f.flg, ed->G_tilt_f.cstr, &(ed->G_tilt) );
  fget_name_cstr_idx(
    ed->accel_kvol_f.name, ed->accel_kvol_f.cstr, &(ed->accel_kvol_f.flg), fi
    );
  par_transval(
    ed->accel_kvol_f.flg, ed->accel_kvol_f.cstr, &(ed->accel_kvol)
    );
  */
  chk_data("}INCIDENT_BEAM", fi);

  /* BEAM_SEARCH block */
  chk_data("BEAM_SEARCH{", fi);
  fget_ed_beams(ed, fi);
  chk_data("}BEAM_SEARCH", fi);

  /* CALC_KXY_RANGE block */
  chk_data("CALC_KXY_RANGE{", fi);
  ed->calc_range_flag = fget_i(fi);
  ed->calc_type = fget_i(fi);
  if(
    ed->calc_type == 0 || ed->calc_type == 1 ||
    ed->calc_type == 2 ||  ed->calc_type == 3
    ){
    ed->calc_numh = fget_i(fi);
    fget_2d( &(ed->calc_hbgn), &(ed->calc_hend), fi );
    fget_2d( &(ed->calc_gbgn), &(ed->calc_gend), fi );
  }
  else{
    mb_fprintf(stdout, "$$ Invalid calc_type=%d. Abort.\n", ed->calc_type);
    exit(EXIT_FAILURE);
  }
  chk_data("}CALC_KXY_RANGE", fi);

  chk_data("SHIFTSCAN_WIDTH{", fi);
  *num_shiftscan = fget_i(fi);
  fget_2d( shiftscan_bgn, shiftscan_end, fi );
  chk_data("}SHIFTSCAN_WIDTH", fi);

  /* INTENSITY_DATA block */
  chk_data("INTENSITY_DATA{", fi);
  fget_1s( ed->intensity_file, fi );
  if( (fint = fopen(ed->intensity_file, "r")) == NULL ){
    mb_fprintf(stdout, "$$ Cannot open %s. Abort.\n", ed->intensity_file);
    exit(EXIT_FAILURE);
  }

  nt = ed->num_thickness = ei->num_thickness = fget_i(fi);
  tmp_nt = fget_i(fint);
  mb_fprintf(stdout, "$$ nt, tmp_nt = %d, %d\n", nt, tmp_nt);
  if( nt != tmp_nt ){
    mb_fprintf(stdout,
      "$$ num_thickness = %d is not equal to int-file's = %d. Abort.\n",
      nt, tmp_nt
      );
    exit(EXIT_FAILURE);
  }

  ed->scale_factor = new_vector(double, nt);
  chk_vector(ed->scale_factor);
  ed->scale_factor_f = new_vector(FIT_CTRL, nt);
  chk_vector(ed->scale_factor_f);
  ed->thickness = new_vector(double, nt);
  chk_vector(ed->thickness);
  ed->thickness_f = new_vector(FIT_CTRL, nt);
  chk_vector(ed->thickness_f);
  ed->thickwidth = new_vector(double, nt);
  chk_vector(ed->thickwidth);
  ed->thickwidth_f = new_vector(FIT_CTRL, nt);
  chk_vector(ed->thickwidth_f);
  ed->num_avrpix = new_vector(int, nt);
  chk_vector(ed->num_avrpix);/*dummy*/
  ed->gaussian_par = new_vector(double, nt);
  chk_vector(ed->gaussian_par);/*dummy*/
  ed->avrpix_wt = new_tensor_element(nt);
  chk_tensor(ed->avrpix_wt);/*dummy*/

  /* ng: number of reflections of experimental data */
  ng = ed->num_rg = ei->num_rg = fget_i(fi);
  tmp_ng = fget_i(fint);
  if( ng != tmp_ng ){
    mb_fprintf(stdout, 
      "$$ Num. of reflex. = %d is not equal to the int file's = %d. Abort.\n",
      ng, tmp_ng
      );
    exit(EXIT_FAILURE);
  }
  chk_data("}INTENSITY_DATA", fi);

  ei->obs_rg = new_vector(HKL, ng);         chk_vector(ei->obs_rg);
  ed->g = new_vector(double, ng);           chk_vector(ed->g);
  ed->art_weight = new_vector(double, ng);  chk_vector(ed->art_weight);

  ed->H_shift = new_matrix(double, ng, nt);     chk_matrix(ed->H_shift);
  ed->G_shift = new_matrix(double, ng, nt);     chk_matrix(ed->G_shift);
  ed->H_shift_f = new_matrix(FIT_CTRL, ng, nt); chk_matrix(ed->H_shift_f);
  ed->G_shift_f = new_matrix(FIT_CTRL, ng, nt); chk_matrix(ed->G_shift_f);

  ei->num_data_points = new_vector(int, ng);
  chk_vector(ei->num_data_points);

  ei->obs_H = (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_H);
  ei->obs_G = (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_G);
  ei->obs_intensity	= (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_intensity);
  ei->cal_intensity	= (double ***) new_tensor_element(ng);
  chk_tensor(ei->cal_intensity);
  ei->obs_sig		= (double ***) new_tensor_element(ng);
  chk_tensor(ei->obs_sig);

  for(ig=0; ig<ng; ig++){
    ei->obs_H[ig]			= (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_H[ig]);
    ei->obs_G[ig]			= (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_G[ig]);
    ei->obs_intensity[ig]	= (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_intensity[ig]);
    ei->cal_intensity[ig]	= (double **) new_matrix_element(nt);
    chk_matrix(ei->cal_intensity[ig]);
    ei->obs_sig[ig]			= (double **) new_matrix_element(nt);
    chk_matrix(ei->obs_sig[ig]);
  }

  for(it=0; it<nt; it++){
    /* THICKNESS_BLOCK part */
    chk_data("THICKNESS_BLOCK{", fi);

    chk_data("THICKNESS_SHIFTSCAN{", fi);
    ed->thickness[0] = fget_d(fi);
    chk_data("}THICKNESS_SHIFTSCAN", fi);
    ed->num_avrpix[it] = 0; /* dummy */

    /* REFLEX_SHIFT_PAR block */
    chk_data("REFLEX_SHIFT_PAR{", fi);
    for(ig=0; ig<ng; ig++){
      ei->obs_rg[ig] = fget_hkl(fint);
      tmp_g = fget_hkl(fi);
      if( 
        ei->obs_rg[ig].h != tmp_g.h ||
        ei->obs_rg[ig].k != tmp_g.k ||
        ei->obs_rg[ig].l != tmp_g.l 
        ) {
        mb_fprintf(stdout, "$$ hkl is inconsistent: \n");
        mb_fprintf(stdout, 
          "$$ par:%d %d %d : int:%d %d %d.    Abort.\n",
          tmp_g.h, tmp_g.k, tmp_g.l,
          ei->obs_rg[ig].h, ei->obs_rg[ig].k, ei->obs_rg[ig].l
          );
        exit(EXIT_FAILURE);
      }

      fget_fitpar(&(ed->H_shift_f[ig][it]), &(ed->H_shift[ig][it]), fi);
      fget_fitpar(&(ed->G_shift_f[ig][it]), &(ed->G_shift[ig][it]), fi);
/*      fget_name_cstr_idx(
        ed->H_shift_f[ig][it].name, ed->H_shift_f[ig][it].cstr,
        &(ed->H_shift_f[ig][it].flg), fi
        );
      par_transval(
        ed->H_shift_f[ig][it].flg, ed->H_shift_f[ig][it].cstr,
        &(ed->H_shift[ig][it])
        );
      fget_name_cstr_idx(
        ed->G_shift_f[ig][it].name, ed->G_shift_f[ig][it].cstr,
        &(ed->G_shift_f[ig][it].flg), fi
        );
      par_transval(
        ed->G_shift_f[ig][it].flg, ed->G_shift_f[ig][it].cstr,
        &(ed->G_shift[ig][it])
        ); */

      nd = ei->num_data_points[ig] = fget_i(fint);

      ei->obs_H[ig][it]  = new_vector_element(double, nd);
      chk_vector(ei->obs_H[ig][it]);
      ei->obs_G[ig][it]  = new_vector_element(double, nd);
      chk_vector(ei->obs_G[ig][it]);
      ei->obs_intensity[ig][it] = new_vector_element(double, nd);
      chk_vector(ei->obs_intensity[ig][it]);
      ei->cal_intensity[ig][it] = new_vector_element(double, nd);
      chk_vector(ei->cal_intensity[ig][it]);
      ei->obs_sig[ig][it] = new_vector_element(double, nd);
      chk_vector(ei->obs_sig[ig][it]);

      for(ik=0; ik<nd; ik++){
        fget_4d(
          &(ei->obs_H[ig][it][ik]), &(ei->obs_G[ig][it][ik]),
          &(ei->obs_intensity[ig][it][ik]), &(ei->obs_sig[ig][it][ik]),
          fint
          );
        if(ei->obs_sig[ig][it][ik] <= 0.0){	/* <<<<< */
          mb_fprintf(stdout, "$$ warning !!: sig <= 0.0\n");
        }
      }
      chk_data("RF_END",fint);
    }
    chk_data("}REFLEX_SHIFT_PAR", fi);
    chk_data("}THICKNESS_BLOCK", fi);
    chk_data("THICK_END",fint);
  }
  chk_data("INTENSITY_DATA_END",fint);
  fclose(fint);
}


void mb_shift_scan( FILE *fi, char *infile )
{
  /*
    HG-shift scan:
   */
  CRYSTAL_STRCT_DATA csd;
  FIT_EXP_DATASET *fed;
  FIT_EXP_INTSET  *fei;
  int    num_exp_dataset, ie, its, ig, ng, ik, nk;
  int    num_shiftscan;
  double shiftscan_bgn, shiftscan_end;
  long   ct, total_num_data;
  char   log_outfile[BUFLEN];
  FILE   *fpo;

  input_csd_cstr(&csd, fi, infile);  /* crystal structure data */

  /* ----- Read experimental data ----- */
  chk_data("SHIFTSCAN_CALCULATION_CONDITION{", fi);
  num_exp_dataset = 1;
  chk_data("DATA_SET{", fi);
  fed = new_vector(FIT_EXP_DATASET, num_exp_dataset);  chk_vector(fed);
  fei = new_vector(FIT_EXP_INTSET, num_exp_dataset);   chk_vector(fei);
  ie = 0;
  input_shiftscan_exp_dataset(
    &(fed[ie]), &(fei[ie]), &num_shiftscan, &shiftscan_bgn, &shiftscan_end, fi);
  chk_data("}DATA_SET", fi);
  chk_data("}SHIFTSCAN_CALCULATION_CONDITION", fi);
  fclose(fi);
  constraint_fed(num_exp_dataset, fed);

  dynamical_calc_shiftscan(
    &csd, ie, &(fed[ie]), &(fei[ie]),
    num_shiftscan, shiftscan_bgn, shiftscan_end, infile
    );

  free_fed_vector(num_exp_dataset, fed);
  free_fei_vector(num_exp_dataset, fei);
  free_csd(&csd);
}
#endif
